///////////////////////////////////////////////////////////////////////////
// Canon EOS Shutter-Emulation on Arduino (Nano / Pro Mini, / ...) V3.3
//  for 6D, 50D, 60D, 70D, 80D, 550D+600D, 650D+700D, 1100D, 1200D+1300D, 2000D
//  (EOS M, M3, M10, M100 --> EOS_M-Shutteremulation.ino)
//
//       25.1.2017 - 23.01.2024                       Erwin Lotter
//
// Select the desired camera by uncommenting the corresponding #define:
//
//#define EOS650D_700D		// (650D und 700D identisch)
//#define EOS600D	
//#define eo63a			//  = 600D-Emulator ohne Coil2-Signal, TRIGGER2_MODE undef.!
//#define EOS1100D
//#define EOS1200D_1300D
//#define EOS2000D
//#define EOS6D
//#define EOS6Dii		// 6D Mark 2
//#define EOS6Dii_MIRROR	// 6D Mark 2, nur Spiegel-Emulation
//#define EOS50D
//#define EOS60D
#define EOS70D
//#define EOS80D		// 80D ohne Spiegel-Emulation
//#define EOS80D_VOLLEMU	// auch Spiegel wird emuliert: noch nicht möglich
//#define EOS200D		// nur Entwurf! LV-Verwendung vmtl. nicht möglich!

// Verwendung des Coil 2 Signals aktivieren durch Angabe der Triggerflanke, die Coil 2 löst (= RISING).
//
#define TRIGGER2_MODE	RISING	// Vorgabe: mit Coil 2 (wenn möglich), sonst nur Coil 1 auswerten
//
// Für Modelle, die nur eine Coil haben (EOS M, 1100D, ...) wird diese Einstellung ignoriert.
// Für Modelle mit 2 Coils, von denen aber nur Coil 1 angeschlossen ist, wird auf der 
//  Basis des Coil 1 Signals emuliert, was aber mit Einschränkungen speziell im 
//  LiveView verbunden ist.
// Die 6D und die 80D erzwingen Coil 2, weil Coil 1 bei EnterLV inaktiv bleibt.

#ifdef eo63a
# define EOS600D	      
# undef  TRIGGER2_MODE	    // eo63a hat kein Coil2-Signal
#endif
/****************************************************************************************************
// Aktueller Stand:

  Achtung: Die Emulation ist nur für bestimmte Einstellungen der Kameras geeignet!
	   Die Spiegel-Vorauslösung sollte (sofern verfügbar) abgeschaltet und für den 
	   Live-View der Modus "Leise 1" gewählt sein (Ausnahme: 60D, Leise-Modus aus).
      Während des Einschaltens sollte nicht ausgelöst werden, z.B. durch eine arretierte Fernauslösung.

  6D:
    - Shutter und Spiegel werden emuliert (mit Coil 1 und 2).
    - Auslösen klappt bis 1/2500s, im LV bis 1/1600s, darunter tritt Err30 auf.
    - Auslösen im LV klappt nur im Leise-Modus 1 (Leise-2 hängt, Normal -> Err30). Lässt sich
      evtl. wie bei der 60D dadurch beheben, dass ROT-Signale verzögert werden.
    - Spiegelvorauslösung führt zu Error 20.
    - Abschalten während einer Belichtung führt zu Error 20.
    - Err20 nach Rückkehr aus Standby?
    - Kleine Blendenöffnungen (Abblenden um mehr als xx Stufen, ab Blendenwert 16 und höher bei einem EF 35-80, ab 5.6 beim EF50 1/1.8)
	    führen beim Auslösen zu Error 30. Dabei wird noch ein zu kurz belichtetes Bild aufgenommen.
	    Ab Mai 2022 sollte das behoben sein!
    - Gelegentlich (alle paar hundert Aufnahmen) hatte 6D-2 Error 30 (wie bei der 600D?).
      Tritt aktuell nicht mehr auf: Könnte mit dem Blenden-Problem verbunden sein.

  6Dii:
    - Shutter und Spiegel-Lichtschranke werden emuliert, der Spiegelmotor muss aber angeschlossen bleiben.
    - Auslösen klappt bis 1/2500s, im LV bis 1/1600s, darunter tritt Err30 auf.
    - Kleine Blendenöffnungen sind möglich.
    - Leise, schnelle und Serien-Auslösungen werden (noch) nicht unterstützt.
    - Auslösen im LV geht schnell (< 0.5s Verlust) und ohne Spiegelbewegung!
    - Auslösen außerhalb des LV benötigt Belichtungsmessung auch im M- und Bulb-Mode!!!

  50D: Ok, auch im LV und mit Spiegelvorauslösung. Shoot im Leise-Modus 1 und 2 für
       Zeiten >= 1s (oder Bulb, egal wie lang) erzeugt Error 20.

  60D: Mit Coil2 (Leise-Modus muss AUS sein? Oder doch Leise1?)

  70D: Mit Coil2, Leise-Modus 1.
    - Auslösen klappt bis t >= 1/2000s auch im LV, darunter tritt Err30/Err20 auf.
  
  80D: Shutter ok für t >= 1/2500, auch im LV-Leise1 aber nicht im Leise-Aufnahmen-Modus.
       Bei sehr kleinen Blenden tritt bei LV-Start Err30 auf. Vmtl. wie bei der 6D zu beheben, s. "80D_BLENDE"!
       Spiegelemulation gibt Err20 bei Shoot und beim Beenden des LV, Shoot im LV ist aber ok.
       Deshalb muss der Spiegelmotor mit seinen Lichtschranken angeschlossen bleiben.
       Wie bei der 6Dii wäre wohl auch eine Emulation der Spiegel-Lichtschranken möglich,
        was deren Verschleiß vermeiden würde.
    - Auslösen im LV geht schnell (< 0.5s Verlust) und ohne Spiegelbewegung!
    - Auslösen außerhalb des LV benötigt Belichtungsmessung auch im M- und Bulb-Mode!!!

  200D: Nur Entwurf! LV-Verwendung vmtl. nicht möglich!

  600D:
    - ohne Coil 2 ist fast alles ok.
    - mit Coil 2 geht alles - etwas ist aber unklar (s. "??600D").
      ACHTUNG: In der eo63a (nur Coil 1) lassen sich keine Zeiten < 1/200 realisieren: Kamera begrenzt!
      ACHTUNG: Das c2_H Timing des Main -13 passt nicht zu den früher gemessenen Werten.
      Ist vermutlich Folge der emulierten ROT- und OPT-Signale!
    - Kleine Blendenöffnungen (Abblenden auf > F11 beim EF50 1/1.8) führten für
      die eo63a (nur Coil 1) beim Auslösen zu Error 30, auch im Bulb Mode - aber nicht im LV.
      Anders als bei der 6D wird aber keine falsche Aktion gewählt, die Kamera
      erwartet scheinbar ein zusätzliches Signal. 
      Ist behoben durch ein zusätzliches OPT2 Lo,Hi in c1_L_Start. 
      (Unklar ob das Problem auch mit Coil 2 auftritt.)
    - Gelegentlich (alle paar hundert Bilder) Error 30 (bei der 600D-3?).
      Bei der 6D-2 wurde das vmtl. durch das Blenden-Problem verursacht, s. dort "Blendenwerte"
       
  700D:
    -  mit Coil 2: Die Zeiten 1/4000 und 1/3200 ergeben Err 30, Rest ist ok
    - ohne Coil 2: Auslösen im LV klappt erst für t > 1/60s.

  1100D: Außerhalb des LV Err30 für t <= 1/500s! (Shoot im LV klappt auch in der alten Version).
  1200D: Außerhalb des LV Err30 für t <= 1/200s! (Shoot im LV klappt auch in der alten Version).
  1300D: Vmtl. wie 1200D oder 2000D.
  2000D: Ok in allen Betriebsarten.
*/

//#define DEBUG_SLOW    // 	ToggleDebugLED() aktivieren, evtl. 64x langsamer
//#define DEBUG_BLINK			// BlinkLED() aktivieren
//#define SERIAL_DEBUG  115200  // undef oder Baud rate

// This Canon EOS shutter-emulation is inspired by the emulator presented by
//  Hendrik Beijeman (http://www.phym.nl/wp/?p=367).
//
// This sketch should be uplaoded with a programmer and replace the Arduino
//  boot code in order to reduce the boot delay. 
// For testing, the sketch can be uploaded as a sketch only. Then, the arduino
//  power supply voltage must be supplied independently at least one second before
//  the camera is powered up. Otherwise, the camera may hang on startup!
// Alternatively, channels which should be low on startup can be pulled down by
//  a 2k resistor, which takes this job until the Arduino output gets active.
//  With this, it is sufficient to run the emulation code as a simple sketch.

// Signalpegel im Ruhezustand (für Setup()):
// (Allgemeine Vorgabewerte, die für einzelne Modelle überschrieben werden.)
#define	ROTX_INIT	LOW
#define	ROT1_INIT	LOW
#define	ROT2_INIT	HIGH
#define	ROT3_INIT	HIGH
#define	ROT4_INIT	HIGH
#define	MR1_INIT	HIGH
#define	MR2_INIT	LOW
#define	MR3_INIT	HIGH	  // ovrrd für 50D
// Die OPT1,2-Startpegel werden ohne Def. als HIGH angenommen

#if defined(EOS6Dii_MIRROR)
# define EOS6Dii	// nur Spiegel der 6Dii: fast wie 6Dii 
#endif
#if defined(EOS6Dii)
# define EOS6D		// 6D und 6Dii unterscheiden sich nur marginal
#endif

#if defined(EOS6D)

// Anschluss-Defs.:
//  Int. Bez.| Arduino-Dx // Kamera-Pin
#define COIL2     2     // 9
#define COIL1     3     // 8
#define OPT1      4     // 10: Opto-feedback from shutter coil 1
#define OPT2      8     // 11: Opto-feedback from shutter coil 2
#define ROT1      7     // 2: Rotary feedback 1
#define ROT2      5     // 3: Rotary feedback 2
#define ROTX      6     // 1: Rotary feedback X
#define MR1       9     // 2: Mirror Rot 2 (M2)
#define MR2       10    // 4: Mirror Rot 4 (M4)
#if defined(EOS6Dii_MIRROR) // Belegung geändert:
#define MR1       7     // Dn
#define MR2       8    // Up (von innen)
#define OPT2      9     // 11: Opto-feedback from shutter coil 2
#define ROT1      10     // 2: Rotary feedback 1
#define OPWR	  OPT1  // Opt-Power Input statt Opt1 Output
#endif

#if defined(EOS6Dii)
#define MDN       MR1   //  : Mirror OptDn
#define MUP       MR2   //  : Mirror OptUp
#else
#define MR3       11    // 5: Mirror Rot 5 (M5)
#endif

#endif

#if defined(EOS60D)

// Anschluss-Defs.:
//  Int. Bez.| Arduino-Dx // Kamera-Pin
#define COIL2     2     // 5
#define COIL1     3     // 4
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      8     // 9: Opto-feedback from shutter coil 2
#define ROT2      5     // 2: Rotary feedback 2
#define ROT3      6     // 3: Rotary feedback 3
#define ROT1      7     // 1: Rotary feedback 1
#define MR1       9     // 5: Mirror Rot S5
#define MR2       10    // 4: Mirror Rot S4
#define MR3       11    // 6: Mirror Rot S6
#define S5 MR1 
#define S4 MR2 
#define S6 MR3 

#undef	ROT1_INIT
#define	ROT1_INIT	HIGH
#undef	ROT2_INIT
#define	ROT2_INIT	LOW
#undef	ROT3_INIT
#define	ROT3_INIT	LOW

#endif

#if defined(EOS70D)

// Anschluss-Defs.:
// Int. Bez.| Arduino-Dx// Kabel-Pin
#define COIL1     2     // 4
#define COIL2     3     // 5
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      5     // 9: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 2: Rotary feedback 2
#define ROT3      8     // 3: Rotary feedback 3

#define MR1       9     // 3: Mirror Rot M1
#define MR2       10    // 2: Mirror Rot M2
#define MR3       11    // 1: Mirror Rot M3

#undef	ROT2_INIT
#define	ROT2_INIT	LOW

#endif

#if defined(EOS80D)

// Anschluss-Defs.:
// Int. Bez.| Arduino-Dx// Kabel-Pin
#define COIL1     2     // 4
#define COIL2     3     // 5
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      5     // 9: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 2: Rotary feedback 2
#define ROT3      8     // 3: Rotary feedback 3

#define M5        9     // 28: Mirror M5
#define M4        10    // 27: Mirror M4
#undef	ROT2_INIT
#define	ROT2_INIT	LOW
#if defined (EOS80D_VOLLEMU)	    // auch Spiegel wird emuliert:
#define MR1       M5    //  Pin nur in Spiegel-Emu als Ausgang def.
#define MR2       M4    //   ..
#define MOTORDRV  11    // 25: Mirror Motor Drive M3
# if defined (MOTORDRV)	
unsigned long stime;
# endif
#endif // EOS80D_VOLLEMU

#endif

#if defined(EOS50D)

// Anschluss-Defs.:
//  Int. Bez.| Arduino-Dx // Kamera-Pin
#define COIL2     2     // 4
#define COIL1     3     // 6
#define OPT1      4     // 11: Opto-feedback from shutter coil 1
#define OPT2      5     // 14: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 3: Rotary feedback 2
#define MR1       8     // M3: Mirror Rot 1
#define MR2       9     // M4: Mirror Rot 2
#define MR3       10    // M5: Mirror Rot 3

#define	MR3_INIT  LOW	// abweichend von den Voreinst. für die 6D

// Zwischen den Shutter-Anschlüssen 9 und 10 hängt ein Temperatursensor, vermutlich ein RTC
//  mit einem Widerstand von 1.2 kOhm bei RT. Ohne diesen meldet die Kamera, die interne 
//  Temperatur sei zu hoch. Mit einem Ersatzwiderstand von 1k2 ist alles gut.
//
#endif

#if defined(EOS600D)

// Anschluss-Defs.:
//  Int. Bez.| Arduino-Dx // Shutter-Pin
#define COIL2     2     // P1
#define COIL1     3     // P3
#define OPT1      4     // P4: Opto-feedback from shutter coil 1
#define OPT2      9     // P7: Opto-feedback from shutter coil 2
#define ROT4      5     // P11: Rotary feedback 4
#define ROT1      6     // P12: Rotary feedback 1
#define ROT2      7     // P8:  Rotary feedback 2
#define ROT3      8     // P9:  Rotary feedback 3

#endif

#ifdef EOS650D_700D
//  Int. Bez.| Arduino-Dx // Shutter-Pin
#define COIL2     2     // P1
#define COIL1     3     // P3
#define OPT1      4     // P4: Opto-feedback from shutter coil 1
#define OPT2      9     // P7: Opto-feedback from shutter coil 2
#define ROT4      5     // P13: Rotary feedback 4
#define ROT1      6     // P14: Rotary feedback 1
#define ROT2      7     // P10: Rotary feedback 2
#define ROT3      8     // P11: Rotary feedback 3
#define ROTX     10     // P9:  Rotary feedback X

#endif 

#if defined(EOS1100D) || defined(EOS1200D_1300D) || defined(EOS2000D)
// P2 = GND (Return GND for phototransistor/diode)
#define COIL1     3     // P7
#define OPT1      4     // P1: Opto-feedback from shutter
#define ROT4      5     // P8: Rotary feedback 4
#define ROT1      6     // P9: Rotary feedback 1
#define ROT2      7     // P10: Rotary feedback 2
#define ROT3      8     // P12: Rotary feedback 3

// Zur Erkennung des LV wird geprüft, ob die Coil Haltedauer in einem bestimmten Limit liegt.
// Dieses Limit wird aber auch für sehr kurze Belichtungen unterschritten (1100D: t <= 1/500s,
//  1200D: t <= 1/200s), was dann zu einem Err30 führt!
// Zwar sind solche Zeiten nicht sinnvoll, aber da die Kameras auch ohne Erkennung des LV
//  alle Funktionen korrekt ausführen, kann auf die Erkennung auch verzichtet werden:
//
#define OHNE_LV_ERKENNUNG

#if defined(OHNE_LV_ERKENNUNG)
# define COIL_HOLD_ENTER_LV  90	  // wird auch von kürzesten Bel. nicht unterschritten
#else
# if defined(EOS1100D)
#  define COIL_HOLD_ENTER_LV  103
# elif defined(EOS2000D)
#  define COIL_HOLD_ENTER_LV  108  // 108.5
# else
#  define COIL_HOLD_ENTER_LV  107
#endif
#endif

#endif

///////////////////////////////////////////////////////////////////////////////////////////////
// Verwendung der Coils als Trigger:
// Trigger 1 ist für alle Modelle zwingend, Trigger 2 muss
//  modellspezifisch mit TRIGGER2_MODE (--> 700D) aktiviert werden.
// Die 6D und 80D erzwingen Coil 2, weil Coil 1 bei EnterLV inaktiv bleibt.
//
#if defined(EOS6D)  ||  defined(EOS80D)
#define TRIGGER1_MODE RISING	  // von Coil 1 nur Stopp-Flanke auswerten
#undef  TRIGGER2_MODE
#define TRIGGER2_MODE CHANGE	  // Coil-Aktivierung an Coil 2 festmachen
#else
#define TRIGGER1_MODE CHANGE
# if defined(EOS1100D) || defined(EOS1200D_1300D) || defined(EOS2000D)
# undef TRIGGER2_MODE		  // nicht vorhanden
# endif
#endif

#define TRIGGER1 COIL1
#define TRIGGER1_ACTIVE LOW
#define TRIGGER2 COIL2
#define TRIGGER2_ACTIVE LOW

//////////////////////////////////////////////////////////////////////////////
// Ergebnis eines Trigger-Ereignisses:
//
#define evNone        0
#define evCoil1Start  1
#define evCoil1Stop   2
#define evCoil2Stop   3

volatile int triggerEvent;    // Flag für Trigger-Flanken
int prevEvent;
unsigned long triggerTime;		// Trigger-Zeitpunkt (von micros()

struct sAction {
  unsigned long time;
  unsigned int  pin;
  unsigned int  value;
};

#define ms *1000L       		// Zeit in Millisekunden in die Basiseinheit (derzeit µs) umrechnen. Achtung: ggf. Klammern!
inline sAction* SelectFinishAction(unsigned long dt) __attribute__((always_inline));


void ToggleLED()
{
  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));   // LED umschalten
}
#if defined(DEBUG_SLOW)
#define ToggleDebugLED()	ToggleLED()
#define DebugSlowdown(t)	((t))// >> 6)        // /64: 10ms -> 0.64s
#else
#define ToggleDebugLED()
#define DebugSlowdown(t)	(t)
#endif

#if defined(DEBUG_BLINK)
int blinkCount;
void BlinkLED(long n);
bool DoBlinkLED();
#else
inline void BlinkLED(long) {}
inline bool DoBlinkLED() { return true; }
#endif

#if defined(SERIAL_DEBUG)
#define PRINT(a)    Serial.print(a)
#define PRINTLN(a)  Serial.println(a)

void print_us(long usecs) {
  char bf[10];
  int millsecs = usecs/1000;
  if (millsecs > 99) {
    sprintf(bf, "%dms ", millsecs);
  }
  else {
    strcpy(bf, "  ms ");
    bf[0] = millsecs / 10 + '0';
    bf[1] = millsecs % 10 + '0';
  }
  PRINT(bf);
}

inline void StartSerial() 
{
  static int isinit = 0;
  if (isinit == 0) {
    isinit = 1;
    Serial.begin(SERIAL_DEBUG);
    PRINTLN("");
    PRINTLN("Starte ...");
  }
}

#else
#define PRINT(a)
#define PRINTLN(a)
void print_us(long) {}
inline void StartSerial() {}
#endif


// Werte für sAction::value (zusätzlich zu HIGH und LOW)
#define STOP_VAL	2
#define HALT_VAL   3
#define LONGWAIT_VAL	4
#define BLINK_VAL   5
#define STOP_ACTION	{0, 0, STOP_VAL}    // ACHTUNG: Mit STOP_ACTION wird der Sleep-Mode gestartet, in dem die Zeitzählung anhält!!!
#define HALT_ACTION	{0, 0, HALT_VAL}
#define LONGWAIT_ACTION	{0, 0, LONGWAIT_VAL}
#define BLINK_ACTION	{0, 0, BLINK_VAL}   // toggled LED, wenn DEBUG_BLINK definiert ist: Nicht als 1. Eintrag!

sAction* actionPtr;

#include <avr/sleep.h>

#ifdef EOS650D_700D	/*////////////////////////////////////////////////////////////////////////

		Startpunkt (stromlos stabil):
		 Innerer Curtain (iC) ist oben (= geschlossen), Anker an der unteren Coil (C2)
		 Äußerer Curtain (aC) ist oben (= geöffnet), Anker an der oberen Coil (C1)

		Beim Auslösen dreht der Motor den aC über den Ruhepunkt hinaus, er wird aber von C1 gehalten.
		Der Spiegel wird danach ausgelöst (hat keine Coil!) und klappt hoch.
		Vmtl. dreht der Motor weiter bis der iC ebenfalls ausgelöst, aber noch von C2 gehalten ist.
		C2 gibt dann iC frei, der (nach unten) öffnet, Belichtung beginnt.
		C1 gibt aC nach T frei, aC schließt (nach unten). Achtung: Der Auslöser dafür ist nicht die
			Freigabe der C2, sondern das optische Signal beim Öffnen des iC! D.h., bei der Emulation
			nur auf Basis von C1 muss OPT2_H nach der vorgegebenen Zeit generiert werden, sonst
			wird C1 nicht freigegeben!

		Abschließend zieht der Motor den Spiegel nach unten und spannt ihn vor.
		Danach wird der iC hochgezogen (Anker bis C2) und zum Schluss der aC (Anker bis C1).

		Die Coil-Versorgung mit 4.7V liegt (anders als bei 600D und 1100D) permanent an und
		 könnte als Arduino-Versorgung dienen. Allerdings kommt sie wohl beim Start zu spät und
		 China-Pro_Mini brauchen möglicherweise auch zu viel Strom.

		Für den Anschluss des Emulators kann das Kabel eines 1100D AF-Sensors (mit Testpunkten!)
		 verwendet werden.

		Regulärer Ablauf (nicht in LV) incl. Start und Restart des LV:
		------------------------------------------------------------------
		1.) Coil 1 --> LOW startet c1_L_Start. Stoppt nach 48 ms, oder nach:
		2.) Coil 2 --> HIGH: Kommt um die Öffnungszeit vor Coil 1 HIGH
				a) Delay < 70ms	(= 68ms) startet c2_H_FinishShoot, das OPT setzt und dann auf Coil1 HIGH wartet.
				b) Delay >= 70ms (= 72ms) startet c2_H_EnterLV, das ein paar Signale korrigiert und stoppt.
		3.) Coil 1 --> HIGH: Sucht in der lfd. Tabelle LONGWAIT und geht weiter bzw. stoppt wenn b).

		Auslösung im LV:
		-----------------
		1.) Coil 1 --> LOW erkennt LV an der zuletzt durchlaufenen Tabelle (c2_H_FinishLVShoot oder c2_H_EnterLV)
				und startet c1_L_StartInLV.
		2.) Coil 2 --> HIGH: Kommt um die Öffnungszeit vor Coil_1_HIGH oder zeitgleich dazu für Leave LV.
				a) Delay < 46ms	(= 43ms, s.u.) startet c2_H_LeaveLV.
				b) Delay < 60ms (= 50ms, s.u.) startet c2_H_FinishLVShoot.
		3.) Coil 1 --> HIGH: Tut nix bei c2_H_LeaveLV, sonst wird LONGWAIT beendet (zuvor ggf. gesucht).
		4.) Coil 1 --> LOW bringt den Shutter (nach c2_H_FinishLVShoot) wieder in den LV-Zustand. Das weitere
				Timing ist fast identisch mit c2_H_EnterLV.

		Wird die Dauer der Pulse auf Coil 2 wird durch das Timing der ROT-Signale beeinflusst?


		// Komplettes Timing der 									".." = liegt schon an
		//  regulären Auslösung,									"x/y" = T / Bulb
		//  getriggert von Coil-1 LOW							"-" = entfällt
		sAction c1_L_Start[] = {	 // EnterLV	 Re-EnterLV	ShootLV		LeaveLV
			{0 ms,     OPT1, LOW  }, //		0					0					0				 -6 ???
			{0 ms,     OPT2, LOW  }, //		0					0					-					-
			{11 ms,    ROT1, HIGH }, //		12				12				..				..
			{12 ms,    ROTX, HIGH }, //		14				14				16				16
			{22 ms,    ROTX, LOW  }, //		27				30
			{32 ms,    ROT2, LOW  }, //		-					-					16				16
			{32 ms,    ROTX, HIGH }, //		-					-
			{48 ms,    ROT2, HIGH }, //		-					-					36/34			34
			{48 ms,    ROT3, LOW  }, //		-					-					36/35			34
			LONGWAIT_ACTION,
			// 68ms		 COIL2 HIGH					73				76				50				43				OPT1 wird hier Hi bis kurz vor C2_H wg. LED-Off
			//  +t     COIL1 HIGH					 +6.4			 +6.4			 +t				+0
			{ +6.4 ms, OPT2, HIGH }, //		 +6.4			 +6.4			..				..
			{ +t+4,    OPT1, HIGH }, //		 +6				 +6				+t+4			+4
			{ +t+5,    OPT1, LOW  }, //		-					-					+t+6			+6
			{ +t+7,    OPT1, HIGH }, //		-					-					+t+8			+7
			{ +t+18,   ROT3, HIGH }, //		-					-					+t+18			+16
			{ +t+47,   ROT4, LOW  }, //		-					-					+t+45			+43
			{ +t+91,   ROT4, HIGH }, //		-					-				+t+90/86		+82
			{ +t+91,   ROT1, LOW  }, //		-					-				+t+90/86		+82
			{ +t+111,  ROTX, LOW  }, //		-					-				+t+108/103	+98
			//																							+t+538 folgt C1_L für Re-EnterLV
			STOP_ACTION,
		};

		////////////////////////////////////////////////////////////////////////////////////////*/

		// Dauer der Coil 1/2 Pulse in ms:
		// Enter LV: 79ms/73ms, Re-Enter LV 82ms/76ms
#define HiLIMIT_C1_ENTER_LV	((82+2) ms)
//#define HiLIMIT_C1_REENTER_LV	((82+2) ms)
#define LoLIMIT_C2_ENTER_LV	((73-3) ms)
#define HiLIMIT_C1_LEAVE_LV	((43+2) ms)			// C1 und C2 brauchen bei LeaveLV 43 ms 
#define HiLIMIT_C2_LEAVE_LV	((43+2) ms)			
#define HiLIMIT_C2_LVSHOOT	((50+9) ms)

#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)
#define IsInWindow_LVShoot(dt)   (dt < HiLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C1_ENTER_LV)
#define IsInWindow_ReEnterLV(dt) IsInWindow_EnterLV(dt)


// Sequenz für reguläres Auslösen bis Coil 2 abschaltet:
//
sAction c1_L_Start[] = {			// Coil aktiv
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{11 ms,  ROT1, HIGH }, // 2
	{12 ms,  ROTX, HIGH }, // 3
	{22 ms,  ROTX, LOW  }, // 4
	{32 ms,  ROT2, LOW  }, // 5
	{32 ms,  ROTX, HIGH }, // 6
	{48 ms,  ROT2, HIGH }, // 7
	{48 ms,  ROT3, LOW  }, // 8
	{79 ms,  OPT2, HIGH }, // OPT2 geht 6 ms nach C2_H Hi. Wenn nur C1 triggert, muss O2_H für EnterLV (und für den LongMode?) hier gesetzt werden.
			       //  Andernfalls wartet die Kamera mit dem C1_H (mehr als 300 ms) und streikt. 
			       //  Wenn O2_H zu früh kommt (75 ms) klappt EnterLV nicht.
	HALT_ACTION,
};

/* Die Sequenz für den Start des LV (bis Coil 1 oder 2 Hi wird) hat durch das
//  Stoppen des Motors ein leicht abweichendes Timing. Zum Glück akzeptiert
// die 700D auch die deutlich abweichende Sequenz für reguläres Auslösen.
// (Der Fehler ohne C2 wird auch nicht dadurch verursacht.)
//
sAction c1_L_Start[] = {
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{12 ms,  ROT1, HIGH }, // 2
	{14 ms,  ROTX, HIGH }, // 3
	{27 ms,  ROTX, LOW  }, //
	{79 ms,  OPT2, HIGH },

	HALT_ACTION,
};
// Da Anfangs nicht erkannt werden kann, welche Aktion die Kamera vorhat,
//  wird stattdessen c1_L_Start ausgeführt. Die 700D akzeptiert aber das
// Timing für reguläres Auslösen und stört sich auch nicht daran,
//  wenn die ROT-Signale wie beim regulären Auslösen folgen.
//*/


// Fortsetzung des regulären Timings nach Coil-1/2 Hi
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2   // Sichere Def für C1 und C2, aber nicht parallel verfügbar
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH },	// -> Curtain öffnet und nach 6 ms geht OPT2 Hi. Wenn bei sehr kurzen 
													//			Belichtungen (<6ms, 1/200) C1 zuvor Hi wird, wird die Aktion vorgezogen.
													//		Tatsächlich ist es auch ok, OPT2_H sofort nach C2_H auszuführen.
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishShoot[] = {
//#endif	
#define c1_H_FinishShoot (c2_H_FinishShoot + 2)	
	// Coil 1 wird inaktiv: für Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geöffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt, deshalb hier nachgeholt!
	{0 ms,    OPT2, HIGH },	// ohne Coil 2 bei kurzen Zeiten noch nicht ausgeführt
	{4 ms,    OPT1, HIGH }, // 11 (Offset 78 abgezogen)
	{5 ms,    OPT1, LOW  }, // 12
	{7 ms,    OPT1, HIGH }, // 13
	{18 ms,   ROT3, HIGH }, // 14
	{47 ms,   ROT4, LOW  }, // 15
	{91 ms,   ROT4, HIGH }, // 16
	{91 ms,   ROT1, LOW  }, // 17
	{111 ms,  ROTX, LOW  }, // 18
	STOP_ACTION,
};

// Sequenz zum Auslösen im oder Beenden des LV. Wird passend fortgesetzt, wenn Coil 2 abschaltet.
// Das Timing ist schneller als beim normalen Auslösen, das für LeaveLV zu Error30 
//  führen würde und auch nicht ausreichend angepasst werden kann.
// 
// Das normale Timing wird für Auslösen im LV akzeptiert, wenn OTP2 Lo gesetzt wird.
//  -> Wie ist das Coil2-Timing dann???
//  Später noch Hi an OPT2 gibt für beide Fälle Fehler!
//
sAction c1_L_StartInLV[] = {
	{ 0 ms,  OPT1, LOW  },
	//unnötig:
	{ 0 ms,  OPT2, HIGH },	// (OPT2 bleibt Hi)
//{ 0 ms,  OPT2, LOW  },
	{ 0 ms,  ROT1, HIGH }, // unnötig: R1 ist schon Hi

	{16 ms,  ROTX, HIGH }, // statt 14, das regulär folgende LOW,HIGH entfällt
	{16 ms,  ROT2, LOW  }, // statt 32 im Nicht-LV

	{34 ms,  ROT2, HIGH }, // statt 48 im Nicht-LV
	{ 0 ms,  ROT3, LOW  },
	// Früheres/späteres ROT2_H, ROT3_L verschiebt C2_H im LeaveLV entsprechend!!!
	// C2_H im ShootLV bleibt bei ca. 51, aber:
	/* Wenn hier OPT1_H_L_H so eingefügt wird, löst das C2_H im ShootLV schon bei 45 ms aus,
	//  das damit an die 43 ms des C2_H in LeaveLV heranrückt! Die Limits müssen dann sehr eng gesetzt werden!
	{ 43 ms,  OPT1, HIGH},	// (Versuch, LeaveLV ohne Coil 2 korrekt zu timen)
	{ 44 ms,  OPT1, LOW},
	{ 46 ms,  OPT1, HIGH},
	//*/
		HALT_ACTION,
};


// Nachdem der LV-Start am Coil-2 Hi Timing erkannt wurde, müssen/können(?) 
//  die ROT-Pegel noch korrigiert werden:
//
sAction c2_H_EnterLV[] = {
	{0 ms,    ROTX, LOW  }, // war schon zu weit! Wieder zurück
	{0 ms,    ROT3, HIGH }, // ROT3 auch nicht
	{0 ms,    ROT2, HIGH }, // (sollte schon stimmen)
	{6 ms,    OPT2, HIGH },	//???? -> Curtain öffnet und nach 6 ms geht OPT2 Hi (Hätte gar nicht low werden sollen!)
	STOP_ACTION,						// dieser Stand wird dann eingefroren (der innere Curtain hält auch nachdem Coil 1 löst, das OPT1 Signal wird nur Hi, weil die LED aus geht)
};

sAction c1_H_EnterLV[] = {
	{0 ms,    ROTX, LOW  }, // war schon zu weit! Wieder zurück
	{0 ms,    ROT3, HIGH }, // ROT3 auch nicht
	{0 ms,    ROT2, HIGH }, // (sollte schon stimmen)
	{6 ms,    OPT2, HIGH },	// ???? -> Curtain öffnet und nach 6 ms geht OPT2 Hi (Hätte gar nicht low werden sollen!)

//{111 ms,  OPT1, LOW},	// (Ist bei leaveLV schon vor dem Start Lo: Erscheint nur wg. fehlender Coil-Pwr HIGH!?)

	STOP_ACTION,						// dieser Stand wird dann eingefroren (der innere Curtain hält auch nachdem Coil 1 löst, das OPT1 Signal wird nur Hi, weil die LED aus geht)
};

// Nachdem das LV-Ende am Coil-2 Hi Timing erkannt wurde, müssen/können(?) 
//  die ROT-Pegel noch korrigiert werden, die hier früher kommen müssen,
//  aber noch auf dem Stand der reg. Auslösung nach 43 ms sind:
//
sAction c2_H_LeaveLV[] = {
	{0 ms,    ROT2, HIGH }, // ROT2 müsste schon Hi sein!
	{0 ms,    ROT3, LOW  }, //  und ROT3 Lo
	{4 ms,    OPT1, HIGH },	// -> innerer Curtain schließt nachdem der Motor weiter dreht und Coil 1 (zeitgleich mit 2) löst und nach 6 ms geht OPT1 Hi
	{6 ms,    OPT1, LOW  },
	{7 ms,    OPT1, HIGH },
	{16 ms,   ROT3, HIGH }, // dann der (vorgezogene) Rest der ROT..
	{43 ms,   ROT4, LOW  },
	{82 ms,   ROT4, HIGH },
	{82 ms,   ROT1, LOW  },
	{98 ms,   ROTX, LOW  },
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// ohne Coil2: für Coil1 identisch

// Fortsetzung des Timings bei Auslösung im LV 
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH }, 				// -> Curtain öffnet und nach 6 ms geht OPT2 Hi; sollte schon Hi sein, wird benötigt!!!!
	LONGWAIT_ACTION,					// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
// #else
// sAction c1_H_FinishLVShoot[] = {
// #endif	
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 2)	
	// Coil 1 wird inaktiv: für Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	{0 ms,    OPT2, HIGH }, 				// falls überrollt
	{4 ms,    OPT1, HIGH }, // 11 ohne Offset 78
	{5 ms,    OPT1, LOW  }, // 12
	{7 ms,    OPT1, HIGH }, // 13
	{18 ms,   ROT3, HIGH }, // 14
	{45 ms,   ROT4, LOW  }, // 15
	{90 ms,   ROT4, HIGH }, // 16
	{90 ms,   ROT1, LOW  }, // 17
	{108 ms,   ROTX, LOW  }, // 18
	STOP_ACTION,
};

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
  // Coil-2 Timing der 700D:
  // - Das reguläre C2_Hi kommt bei 68 ms, im LV nach 50 ms
  // - Beim LV-Start kommt C2_Hi bei 73 ms (Verzögerung durch das Anhalten des Motors?!)
  // - Am Ende des LV ist der C2 Puls nur 43 ms breit
  //
  //BlinkLED(dt);//45 ms für Shoot!
  if (dt < HiLIMIT_C2_LEAVE_LV) {		// LV-Stop
    return c2_H_LeaveLV;
  }
  else if (IsInWindow_LVShoot(dt)) {		// Auslösen im LV
    return c2_H_FinishLVShoot;
  }
  else if (dt >= LoLIMIT_C2_ENTER_LV) {	// EnterLV
    return c2_H_EnterLV;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    return c2_H_FinishShoot;
  }
    /*/ // Variante, die (auch) die aktuelle Liste mit auswertet:
    if (actionPtr == c1_L_Start) {		// (noch) nicht im LV?
	    if (dt < 70 ms) {		// ca. 68 ms: reguläres Auslösen
		    actionPtr = c2_H_FinishShoot;
	    }
	    else {										// ca. 73 ms: LV-Start oder Restart
		    actionPtr = c2_H_EnterLV;
	    }
    }
    else {
	    if (digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {		// im LV: Coil 1 noch aktiv?
		    actionPtr = c2_H_FinishLVShoot; 					//  dann ist es Shoot
	    }
	    else {
		    actionPtr = c2_H_LeaveLV;					// beim Beenden des LV kommt Coil 1 sofort zurück
	    }
    }
    //*/
}

#endif //EOS650D_700D 
// ====================================================================================================
#ifdef EOS600D

/* Coil-Timing
	Modus	     Coil 1  Coil 2

	Shoot      76+t     76	      
	Enter LV    94      87
	Shoot LV   71+t     71
	Leave LV    40      --

*/
// Coil-2 Timing der 600D-3 
// (Die Blink-Werte des Main-13 in Klammern weichen stark davon ab! Vmtl. hat die
//	Start-Sequenz einen Einfluss auf das Timing.))
// - Das reguläre C2_Hi kommt bei 77 ms (!81!), bei Shoot im LV nach 71(!74!) ms
// - Beim LV-Start kommt C2_Hi bei 85 ms, Re-Enter 88 ms (!68!).
//   (Dauer der 600D-3 Coil 1 Pulse: Enter LV: 91ms, Re-Enter LV 94ms)
// - Am Ende des LV bleibt C2 Hi
// --> Grenzwerte zur Unterscheidung der Funktion bei C1_H
#define HiLIMIT_C1_ENTER_LV	((83+2) ms)			// schneller als im Original, weil von C1_H in c1_L_Start ausgelöst?
#define HiLIMIT_C1_REENTER_LV	((96+2) ms)
#define HiLIMIT_C1_LEAVE_LV	((41+4) ms)			// C1 braucht bei LeaveLV 41 ms 
//#define LoLIMIT_C2_ENTER_LV	((86-3) ms)			// aus 600D-3 Timing! Mit Main-13 68 ms: kürzester Puls
#undef  HiLIMIT_C2_LEAVE_LV									// C2 wird für LeaveLV gar nicht erst LOW
#define HiLIMIT_C2_LVSHOOT	((71+5) ms)			// große Toleranz (+5) erfasst auch Main -13

#define IsInWindow_LVShoot(dt)   (dt < HiLIMIT_C2_LVSHOOT)	// 76
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C1_ENTER_LV)	// 85
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C1_REENTER_LV)	// 98
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)	// 45


sAction c1_L_Start[] = {			// Coil aktiv
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{19 ms,  ROT1, HIGH }, // 2
	{30 ms,  ROT2, LOW  }, // 3
	{53 ms,  ROT2, HIGH }, // 4
	{62 ms,  ROT3, LOW  }, // ca. 12 ms später als bei der 700D (14 ms vor C2_H bei 76 ms)
	//
	// Wenn nur C1 triggert, muss OPT2_Hi für den LongMode hier schon gesetzt werden:
	{83 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi.
	#ifndef TRIGGER2_MODE
	//
	// OPT2 jetzt nochmals zu schalten behebt das Blende-Problem
	//  (das evtl. auch nur ohne Coil2 auftritt).
	// Ist bisher nur ohne Coil2 an der eo63a getestet.
	// (Evtl. erwartet die Kamera bei großen Blenden das OPT2 Hi später!)
	//
	{93 ms,  OPT2, LOW }, // geht 7 ms nach C2_H Hi.
	{123 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi.
	#endif	  
	HALT_ACTION,
};


sAction c1_L_StartInLV[] = {			// Coil aktiv im LV
	{0 ms,   OPT1, LOW  }, // sollte schon anliegen
	//{0 ms,   OPT2, LOW  }, // entfällt
	//{19 ms,  ROT1, HIGH }, // sollte schon anliegen
	//{30 ms,  ROT2, LOW  }, // sollte schon anliegen
	{20 ms,  ROT2, HIGH }, // 4
	{31 ms,  ROT3, LOW  },
	{78 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi. Wenn nur C1 triggert, muss es für den LongMode hier schon gesetzt werden.
	HALT_ACTION,
};


sAction c1_H_EnterLV[] = {			// Coil 1 löst im LV
	{0 ms,    ROT2, LOW }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,    ROT3, HIGH  }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{7 ms,   OPT1, LOW},
	{7 ms,   OPT2, HIGH},	// es scheint egal zu sein welchen Pegel O1 und O2 hier bekommen!
	STOP_ACTION,
};

sAction c2_H_EnterLV[] = {			// identisch für Coil 2
{0 ms,    ROT2, LOW }, //	korrigieren: wurde in c1_L_Start schon gesetzt
{0 ms,    ROT3, HIGH  }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{7 ms,   OPT1, LOW}, 	// bleibt LOW
	{7 ms,   OPT2, HIGH}, //	es scheint egal zu sein welchen Pegel O1 und O2 hier bekommen!
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
sAction c1_H_LeaveLV[] = {		// Coil 1 löst im LV:		
	{ 0 ms,  OPT2, HIGH }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH }, // (Offset 41 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 17 ms, ROT3, HIGH },
	{ 45 ms, ROT4, LOW  },
	{ 96 ms, ROT4, HIGH },
	{ 96 ms, ROT1, LOW  },
	STOP_ACTION,
};

// Fortsetzung des regulären Timings nach Coil-1/2 Hi
// Curtain von Coil 2 hat schon vorher geöffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
//  mehr ausgeführt, deshalb hier nachgeholt!
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH },	// -> Curtain öffnet und nach 6 ms geht OPT2 Hi. Wenn bei sehr kurzen 
													//			Belichtungen (<6ms, 1/200) C1 zuvor Hi wird, wird die Aktion vorgezogen.
													//		Tatsächlich ist es auch ok, OPT2_H sofort nach C2_H auszuführen.
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishShoot[] = {
//#endif	
#define c1_H_FinishShoot (c2_H_FinishShoot + 2)	
	// Coil 1 wird inaktiv: für Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geöffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt, deshalb hier nachgeholt!
	{ 0 ms,  OPT2, HIGH },	// ohne Coil 2 bei kurzen Zeiten noch nicht ausgeführt
	{ 4 ms,  OPT1, HIGH }, // (Offset 79 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 26 ms, ROT3, HIGH },
	{ 57 ms, ROT4, LOW  },
	{106 ms, ROT4, HIGH },
	{107 ms, ROT1, LOW  },
	STOP_ACTION,
};


// ... im LV
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2 (Sichere Def für C1 und C2, aber nicht parallel verfügbar:)
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
//{4 ms,    OPT2, HIGH },	// -> Curtain öffnet (OPT2 war gar nicht LOW)
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishLVShoot[] = {
//#endif	
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	// Coil 1 wird inaktiv: für Coil 1 allein kein LONGWAIT, weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geöffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt, deshalb hier nachgeholt!
	{ 4 ms,  OPT1, HIGH }, // (Offset 79 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 18 ms, ROT3, HIGH },
	{ 45 ms, ROT4, LOW  },
	{ 88 ms, ROT4, HIGH },
	{ 88 ms, ROT1, LOW  },
	STOP_ACTION,
};

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
  // Coil-2 Timing der 600D-3 
  // (Die Blink-Werte des Main-13 in Klammern weichen stark davon ab! Vmtl. hat die
  //	Start-Sequenz einen Einfluss auf das Timing.))
  // - Das reguläre C2_Hi kommt bei 77 ms (81), bei Shoot im LV nach 71(74) ms
  // - Beim LV-Start kommt C2_Hi bei 86 ms (68)
  // - Am Ende des LV bleibt C2 Hi
  //BlinkLED(dt);	// gibt auch im LV den Shoot-Wert
  //BlinkLEDForced(dt);	// gibt im LV den Re-Enter Wert
  if (dt < 70 ms) {					// statt dt>LoLIMIT_C2_ENTER_LV: C2_H kommt schon bei 68 ms, wenn EnterLV mit c1_L_Start eingeleitet wird!
    return c2_H_EnterLV;
  }
  else if (IsInWindow_LVShoot(dt)  	// (700D 50+9 ms, 600D 71+2 ms:) Auslösen im LV
	   &&  actionPtr != c1_L_Start	//20231213:  Hilft nich gegen das Blendenproblem! (6D: normales Shoot ausschließen, das durch große Blendenwerte auf ähnliche Zeiten kommt)
  ) {
    return c2_H_FinishLVShoot;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    return c2_H_FinishShoot;
  }
}

#endif // EOS600D --------------------------------------------

#if defined(EOS1100D) || defined(EOS1200D_1300D) || defined(EOS2000D)

// Die 1100D etc. haben nur eine Coil und einen Vorhang, der beim normalen Auslösen geschlossen,
//  im LiveView geöffnet ist. Beim Auslösen läuft der Motor an und öffnet, gleichzeitig
//  wird die Coil aktiviert. Wenn der Curtain geöffnet ist, erzeugt die Lichtschranke (OPT1)
//  eine Lo/Hi/Lo Folge und der Motor stoppt mit ROT3=LOW. Bis hier ist das Timing identisch
//  für Short, Long und Bulb.
//  Die interne Feder ist jetzt aufgezogen, so dass der Vorhang schließt, sobald die Coil löst.
//  Die Aufnahme beginnt wohl elektronisch durch Beenden der Sensor-Löschfunktion, ausgelöst
//  durch OPT1 (oder ROT3). Sie endet durch Lösen der Coil. Beim Schließen erzeugt OPT1 (nach
//  ca. 5 ms) die umgekehrt Lo/Hi/Lo Folge. Abschließend dreht der Motor wieder bis zur 
//  Startposition mit ROT1=LOW (kurz vor Öffnung).
// Der LV startet etwas langsamer und stoppt schon kurz bevor ROT2 Hi (und ROT3 Lo) wird.
//
// Strategie: 
//  - Wenn die c1_L-Dauer bei COIL_HOLD_ENTER_LV ms liegt, ist der LV-Mode gestartet. Dann wird
//    mit c1_H_EnterLV abgeschlossen und für das nächste c1_L c1_L_StartInLV verwendet.
//  - Wenn die c1_L-Dauer bei 48 ms liegt, ist der LV-Mode verlassen. Dann wird
//    mit c1_H_LeaveLV abgeschlossen und für das nächste c1_L c1_L_Start verwendet.
//  
#define LoLIMIT_C1_ENTER_LV	((COIL_HOLD_ENTER_LV-3) ms)
#define HiLIMIT_C1_ENTER_LV	((COIL_HOLD_ENTER_LV+4) ms)
#define HiLIMIT_C1_LEAVE_LV	((48+5) ms)			// C1 braucht bei LeaveLV 48 ms 

#define IsInWindow_EnterLV(dt)  (dt <= HiLIMIT_C1_ENTER_LV  &&  dt >= LoLIMIT_C1_ENTER_LV)
#define IsInWindow_ReEnterLV(dt) IsInWindow_EnterLV(dt)
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)

#ifdef EOS1100D
sAction c1_L_Start[] = {			// Coil aktiv
	{25850L,  ROT1, HIGH }, // 0
	{34 ms,   OPT1, LOW  }, // 1
	{35100L,  ROT2, LOW  }, // 2
	{39400L,  OPT1, HIGH }, // 3
	{42 ms,   OPT1, LOW  }, // 4
	{62900L,  ROT2, HIGH }, // 5
	{71400L,  ROT3, LOW  }, // 6
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulären Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },	      // Achtung: 5.8 ms führt im LV (ohne Erkennung) z Err30!
	{6 ms,  OPT1, LOW  }, // 8
	{7 ms,  OPT1, HIGH }, // 9
	{34800L, ROT3, HIGH }, // 10
	{71 ms,  ROT4, LOW  }, // 11
	{107 ms,  ROT4, HIGH }, // 12
	{108 ms,  ROT1, LOW  }, // 13
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch früher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{0 ms,  ROT2, HIGH }, // sollte schon erledigt sein
	{0 ms,  ROT3, LOW  }, // fehlt noch, wenn mit c1_L_Start gestartet
	{4 ms, OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{36 ms, ROT3, HIGH }, // 10
	{75 ms,  ROT4, LOW  }, // 11
	{112 ms,  ROT4, HIGH }, // 12
	{112 ms,  ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder für Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder für Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{27 ms,  ROT2, HIGH },
	{37 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


// Nach c1_H_FinishLVShoot (das an t=66ms erkannt wird) kommt eine Re-EnterLV Sequenz
//
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 löst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgeführt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{5500L,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 22 ms, ROT3, HIGH },
	{ 52 ms, ROT4, LOW  },
	{ 89 ms, ROT4, HIGH },
	{ 90 ms, ROT1, LOW  },
	STOP_ACTION,
};

#elif defined(EOS1200D_1300D)

// Die 1200D hat einen sehr ähnlichen Shutter wie die 1100D mit minimal anderem Timing.
// (Beide Kameras akzeptieren beide Timings!)
// Die 1200D fährt im LV den Motor offenbar etwas zurück und erzeugt dabei
//  zusätzliche Signale auf R2, die aber nur für LV-Start abgeprüft werden.
//
sAction c1_L_Start[] = {			// Coil aktiv
	{24 ms,   ROT1, HIGH }, // 0
	{34 ms,   OPT1, LOW  }, // 1
	{36 ms,   ROT2, LOW  }, // 2
	{39 ms,   OPT1, HIGH }, // 3
	{41 ms,   OPT1, LOW  }, // 4
	{56 ms,   ROT2, HIGH }, // 5
	{65 ms,   ROT3, LOW  }, // 6
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulären Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },	      // Achtung: 5.8 ms führt im LV (ohne Erkennung) zu Err30!
	{6 ms,  OPT1, LOW  }, 
	{7 ms,  OPT1, HIGH }, 
	{35 ms, ROT3, HIGH }, 
	{70 ms,  ROT4, LOW  }, 
	{105 ms, ROT4, HIGH }, 
	{105 ms, ROT1, LOW  }, 
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch früher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{4 ms,   OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{37 ms,  ROT3, HIGH }, // 10
	{71 ms,  ROT4, LOW  }, // 11
	{107 ms, ROT4, HIGH }, // 12
	{108 ms, ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder für Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder für Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{28 ms,  ROT2, HIGH },
	{38 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


//
// Nach c1_H_FinishLVShoot kommt eine Re-EnterLV Sequenz
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 löst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgeführt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{ 6 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 18 ms, ROT3, HIGH },
	{ 49 ms, ROT4, LOW  },
	{ 83 ms, ROT4, HIGH },
	{ 84 ms, ROT1, LOW  },
	STOP_ACTION,
};

#else // EOS2000D:

// Die 2000D hat einen sehr ähnlichen Shutter wie ihre Vorgänger mit minimal anderem Timing.
// Wie die 1200D fährt sie im LV den Motor offenbar etwas zurück und erzeugt dabei
//  zusätzliche Signale auf R2, die aber nur für den LV-Start abgeprüft werden!
//
sAction c1_L_Start[] = {			// Coil aktiv
//*
	{24 ms,   ROT1, HIGH }, // 0
	{32 ms,   OPT1, LOW  }, // 1
	{34 ms,   ROT2, LOW  }, // 2
	{36 ms,   OPT1, HIGH }, // 3
	{40 ms,   OPT1, LOW  }, // 4
	{56 ms,   ROT2, HIGH }, // 5
	{54 ms,   ROT2, LOW  }, // hier nicht benötigt, aber für LV
	{61 ms,   ROT2, HIGH }, //  Korrektur
	{62 ms,   ROT3, LOW  }, // 6
/*/ // Timing wie für LV Start: Ist auch für Shoot ok!
  	{40 ms,   ROT1, HIGH }, // 0
	{52 ms,   OPT1, LOW  }, // 1
	{57 ms,   OPT1, HIGH }, // 3
	{59 ms,   ROT2, LOW  }, // 2
	{60 ms,   OPT1, LOW  }, // 4
	{61 ms,   ROT2, HIGH }, // 5
	{64 ms,   ROT2, LOW  }, // hier nicht benötigt, aber für LV
	{67 ms,   ROT2, HIGH }, //  Korrektur: wird aber nicht benötigt
	{67 ms,   ROT3, LOW  }, // 6
//*/	
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulären Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },
	{6 ms,  OPT1, LOW  }, 
	{7 ms,  OPT1, HIGH }, 
	{33 ms, ROT3, HIGH }, 
	{56 ms,  ROT4, LOW  }, 
	{98 ms, ROT4, HIGH }, 
	{99 ms, ROT1, LOW  }, 
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch früher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{4 ms,   OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{34 ms,  ROT3, HIGH }, // 10
	{66 ms,  ROT4, LOW  }, // 11
	{100 ms, ROT4, HIGH }, // 12
	{100 ms, ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder für Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder für Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{26 ms,  ROT2, HIGH },
	{36 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


//
// Nach c1_H_FinishLVShoot kommt eine Re-EnterLV Sequenz
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 löst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgeführt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{ 6 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 17 ms, ROT3, HIGH },
	{ 46 ms, ROT4, LOW  },
	{ 78 ms, ROT4, HIGH },
	{ 79 ms, ROT1, LOW  },
	STOP_ACTION,
};

#endif // EOS2000D
#endif // EOS1100D || EOS1200D_1300D || EOS2000D ###################################

#ifdef EOS6D // oder EOS6Dii

sAction c1_L_Start[] = {			// Coil 2 aktiv
	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, HIGH },
	{0 ms,   ROT1, LOW  },
	{0 ms,   ROTX, LOW  },
#ifdef EOS6Dii
	{ 1 ms,  MDN , LOW  },
	{ 4 ms,  ROT1, HIGH },
	{ 5 ms,  ROTX, HIGH },
	{11 ms,  ROTX, LOW  },
	{15 ms,  ROTX, HIGH },
	{15 ms,  ROT1, LOW  },
	{20 ms,  MUP , HIGH },
	{21 ms,  ROT2, LOW  },
#else
	{ 5 ms,  ROT1, HIGH },
	{ 6 ms,  ROTX, HIGH },
	{ 9 ms,  MR3 , LOW  },
	{ 9 ms,  MR2 , HIGH },
	{11 ms,  MR3 , HIGH },
	{12 ms,  MR1 , LOW  },
	{14 ms,  ROTX, LOW  },
	{17 ms,  ROT1, LOW  },
	{18 ms,  ROTX, HIGH },
	{24 ms,  ROT2, LOW  },
#endif // 6Dii
	HALT_ACTION,		// weiter gehts wenn C2 Hi die Belichtung startet (bei der Spiegelvorauslösung hält der Ablauf hier an)
};


// Start für den LV:  Coil 2 ohne Coil 1 aktiv
//
sAction c2_L_StartForLV[] = {	
	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, HIGH },
	{0 ms,   ROT1, LOW  },
	{0 ms,   ROTX, LOW  },
#ifdef EOS6Dii
	{0 ms,   MDN , LOW  },
	{8 ms,   ROTX, HIGH  },
	{9 ms,   ROT1, HIGH  },
	{18 ms,  MUP , HIGH },
#else
	{8 ms,   ROTX, HIGH  },	// später als bei Shoot!
	{9 ms,   ROT1, HIGH  },	//  dito
	{ 9 ms,  MR3 , LOW  },
	{ 9 ms,  MR2 , HIGH },
	{11 ms,  MR3 , HIGH },
	{12 ms,  MR1 , LOW  },
#endif // 6Dii
	{21 ms,  ROTX, LOW  },  // später als bei Shoot!
	HALT_ACTION,		// weiter gehts mit C2 Hi
};

// Fortsetzung des regulären Shoot-Timings nach Coil-1/2 Hi
// 
// Zuerst wird Coil 2 inaktiv und die Pulsfolge auf Opt2 wird erzeugt.
// Danach wird gewartet, bis Coil-1 Hi das Ende der Belichtung anzeigt und die Opt1-
//  und weiteren Signale ausgegeben werden.
// Bei kurzen Belichtungen ist die Opt2-Folge noch nicht regulär abgearbeitet, wenn
//  C1_H kommt und wird dann kompakt ausgegeben. Das scheint kein Problem zu sein.
// Die 6Dii gibt ab 1/3200s Err30 aus. Dagegen hilft auch nicht, die
//  Opt2-Folge komplett bei 0 ms auszugeben (was generell aber ok ist!).
//  Es hilft auch nicht, die Opt2-Folge nach dem LONGWAIT nochmal auszugeben (was 
//   generell aber auch ok ist (getestet mit der 6Dii)).
//  Das Problem liegt darin, dass C2_H gar nicht mehr bearbeitet wird, wenn C1_H zu
//   früh kommt, weil in trigger1_interrupt() angenommen wird, dass C1_H und C2_H
//   zusammen kommen und LeaveLV anzeigen.
//
sAction c2_H_FinishShoot[] = {	// Coil 2 wird inaktiv
	{3 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 4 ms geht OPT2 Lo. Bei sehr kurzen Belichtungen (<6ms, 1/200) wird die Aktion vorgezogen.
	{5 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{6 ms,   OPT2, LOW },	// (auch Timing 0,0,0 ist ok!)
	LONGWAIT_ACTION,	// hier warten, bis Coil 1 Hi wird und die Belichtung beendet
#define c1_H_FinishShoot (c2_H_FinishShoot + 4)	
	// Coil 1 wird inaktiv: 
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 8 ms,  OPT1, LOW  },
#ifdef EOS6Dii
	{ 18 ms, MUP , LOW  },
	{ 22 ms, ROT2, HIGH },
	{ 23 ms, ROT1, HIGH },
	{ 36 ms, MDN , HIGH }, 
	{ 68 ms, ROT1, LOW  },
	{ 76 ms, ROTX, LOW  },
	{ 76 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 76 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
#else
	{ 9 ms,  MR1 , HIGH }, 
	{41 ms,  MR3 , LOW  },
	{52 ms,  MR3 , HIGH }, 
	{54 ms,  MR2 , LOW  },
	{ 61 ms, ROT1, HIGH },
	{ 62 ms, ROT2, HIGH },
	{108 ms, ROT1, LOW  },
	{116 ms, ROTX, LOW  },
	{116 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{116 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
#endif // 6Dii
	STOP_ACTION,
};

/////////////////////////////////////
// Start nach Shoot im LV:  Coil 2 ohne Coil 1 aktiv
//
sAction c2_L_StartInLVForLV[] = {	
	{8 ms,   ROTX, HIGH  },  // später als bei Shoot!
	{9 ms,   ROT1, HIGH  },
	{21 ms,  ROTX, LOW  },
	HALT_ACTION,		// weiter gehts mit C2 Hi
};


//
// Start in LV: Coil 1+2 aktiv. Wird für LV-Shoot und LV-Stop ausgef.
sAction c1_L_StartInLV[] = {			// Coil 2 aktiviert im LV
	{ 4 ms,  ROTX, HIGH },
	{ 5 ms,  ROT1, LOW  },
	{12 ms,  ROT2, LOW  }, // 7ms nach R1_L
	HALT_ACTION,
};

// Fortsetzung des LVShoot-Timings nach Coil-1/2 Hi
// !!! Klappt auch mit c2_H_FinishShoot: Spiegelsignale werden ignoriert. !!!
//
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
      //{4 ms,    OPT2, HIGH },	// -> Curtain öffnet (OPT2 war gar nicht LOW)
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	{ 4 ms,  OPT1, LOW  },
	{ 6 ms,  OPT1, HIGH },
	{ 7 ms,  OPT1, LOW  },
	{ 58 ms, ROT1, HIGH },
	{ 60 ms, ROT2, HIGH },
	{104 ms, ROT1, LOW  },
	{112 ms, ROTX, LOW  },
	{116 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{116 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	STOP_ACTION,
};


// Beim Start des LV löst Coil 2 schon nach 50 ms (ohne Coil-1 Signal).
// Wenn anhand der C2_L Dauer vom normalen Shoot unterschieden wird, sind die ROT-Signale
//  schon zu weit, weil der Motor im LV-Leise-1 das Schließen noch nicht vorspannt.
//  Das scheint zunächst kein Problem zu sein, aber beim Shoot im oder Stop des LV 
//  gibts Error 20. Die Pegel hier zu korrigieren geht aber nicht!
sAction c2_H_EnterLV[] = {	
	//{0 ms,    ROT2, HIGH}, // korrigieren: wurde in c1_L_Start schon gesetzt
	//{0 ms,    ROT1, HIGH}, // ..
	//{0 ms,    ROTX, LOW }, // ..
	{4 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 4 ms geht OPT2 Lo. Bei sehr kurzen Belichtungen (<6ms, 1/200) wird die Aktion vorgezogen.
	{7 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{8 ms,   OPT2, LOW },
	STOP_ACTION,
};
#define c1_H_EnterLV c2_H_EnterLV

// Nachdem das LV-Ende am Coil-2 Hi Timing erkannt wurde, müssen/können(?) 
//  die ROT-Pegel noch korrigiert werden, die hier früher kommen müssen,
//  aber noch auf dem Stand der reg. Auslösung nach 28 ms sind:
//
sAction c2_H_LeaveLV[] = {
	{ 4 ms,  OPT1, LOW  },
	{ 6 ms,  OPT1, HIGH },
	{ 7 ms,  OPT1, LOW  },
#ifdef EOS6Dii
	{ 25 ms, ROT2, HIGH },
	{ 26 ms, ROT1, HIGH },
	{ 31 ms, MUP , LOW  },
	{ 64 ms, MDN , HIGH }, 
	{ 86 ms, ROT1, LOW  },
	{ 94 ms, ROTX, LOW  },
	{ 94 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 94 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
#else
	{ 9 ms,  MR1 , HIGH }, 
	{41 ms,  MR3 , LOW  },
	{52 ms,  MR3 , HIGH }, 
	{54 ms,  MR2 , LOW  },
	{61 ms,  ROT1, HIGH },
	{62 ms,	 ROT2, HIGH },
	{107 ms, ROT1, LOW  },
	{114 ms, ROTX, LOW  },
	{116 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{116 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
#endif // 6Dii
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// ohne Coil2: für Coil1 identisch

///////////////////////////////////////
/*  6D Coil-Timing (LV im Leise-Modus 1):
	Modus	   Coil 1   Coil 2    6Dii_C2  6Dii_C2   
						R2_L +
	Shoot      52+t-1   52         50        37.5	Verlängert sich für große Blendenwerte!!!  
	Shoot LV   56+t-1   54..56     52.5	 39
	Enter LV    --      50	       38.5	 -
	ReEnter LV  --      36..38     40.5      -
	Leave LV    28      27..28     27        14.5

	C1 wird für (Re-)EnterLV nicht aktiviert. Da C2 immer zeitgleich mit C1 startet
	 wird C2_H als Basissignal verwendet!
*/

// --> Grenzwerte zur Unterscheidung der Funktion bei C1_H:
#ifdef EOS6Dii
#define HiLIMIT_C2_ENTER_LV	((39+1) ms)
#define HiLIMIT_C2_REENTER_LV	((40+2) ms)
#define HiLIMIT_C2_LEAVE_LV	((28+6) ms)	// hängt davon ab, wie schnell der Motor läuft: ~= R2_L + 15ms
#define LoLIMIT_C2_LVSHOOT	((52-1) ms)
#else
#define HiLIMIT_C2_ENTER_LV	((50+1) ms)	// 6Dii: nur 43ms
#define HiLIMIT_C2_REENTER_LV	((38+2) ms)	// 
#define HiLIMIT_C2_LEAVE_LV	((28+6) ms)	// hängt davon ab, wie schnell der Motor läuft: ~= R2_L + 15ms
#define LoLIMIT_C2_LVSHOOT	((54-1) ms)
#endif // 6Dii
#define HiLIMIT_C1_LEAVE_LV	HiLIMIT_C2_LEAVE_LV

#define IsInWindow_LVShoot(dt)   (dt >= LoLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C2_ENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C2_REENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)

// Erste sAction Liste auswählen für ein Coil-Start Ereignis
//
inline sAction* SelectStartAction()
{
  if (digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {   // auch Coil 1 ist aktiv: Auslösen oder LV-Stop
    if (actionPtr == c2_H_EnterLV) {		    //  sind wir im LV?
      PRINTLN("c1_L_StartInLV");
      return c1_L_StartInLV;
    }
    else {
      PRINTLN("c1_L_Start");
      return c1_L_Start;
    }
  }
  else {		  // ohne Coil 1: LV-Start - ist innerhalb nach LVShoot oder LV-ReEnter möglich
    if (actionPtr == c2_H_EnterLV  ||  actionPtr == c2_H_FinishLVShoot) {		  
      PRINTLN("c2_L_StartInLVForLV");
      return c2_L_StartInLVForLV;
    }
    else {
      PRINTLN("c2_L_StartForLV");
      return c2_L_StartForLV;
    }
  }
}

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
//BlinkLED(dt);
  if (dt < HiLIMIT_C2_LEAVE_LV) {		// LV-Stop (27ms)
    PRINTLN("c2_H_LeaveLV");
    return c2_H_LeaveLV;
  }
  else if (IsInWindow_LVShoot(dt)  &&		// Auslösen im LV (56ms)
	   actionPtr != c1_L_Start		//  aber normales Shoot ausschließen, das durch große Blendenwerte auf ähnliche Zeiten kommt!
  ) {
    PRINTLN("c2_H_FinishLVShoot");
    return c2_H_FinishLVShoot;
  }
  else if (dt < HiLIMIT_C2_ENTER_LV) {		// EnterLV (50ms)
//BlinkLED(31 ms);
    PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;			// Shoot (52..55ms)
  }
}

#endif // EOS6D --------------------------------------------

#ifdef EOS80D

sAction c1_L_Start[] = {			// Coil 2 aktiv
        { 0 ms,   M4, HIGH },	  
	{ 6 ms,  ROT1, HIGH },
	{ 7 ms,  ROT2, HIGH },
	{13 ms,  ROT1, LOW  },
	{18 ms,  ROT1, HIGH },
	{22 ms,  M5 , LOW  },
	{26 ms,  ROT2, LOW  },
	{28 ms,  ROT3, LOW  },
	HALT_ACTION,		// weiter gehts wenn C2 Hi die Belichtung startet (bei der Spiegelvorauslösung hält der Ablauf hier an)
};

// Start für den LV:  Coil 2 ohne Coil 1 aktiv
//
sAction c2_L_StartForLV[] = {	
	{ 0 ms,   M4, HIGH},	  //vmtl. nicht nötig oder zu spät
	{13 ms,   ROT1, HIGH  },	// später als bei Shoot!
	{14 ms,   ROT2, HIGH  },	//  dito
	{29 ms,  ROT1, LOW  },  // später als bei Shoot!
	{42 ms,  M5 , LOW  },
	HALT_ACTION,		// weiter gehts mit C2 Hi
};

// Fortsetzung des regulären Shoot-Timings nach Coil-1/2 Hi
//
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{0 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 2/3/4 ms geht OPT2 L/H/L. Bei sehr kurzen Belichtungen (< 1/250) --> Err30. Mit Timing 0/0/0 klappts bis 1/3200, im LV auch weiter.
	{0 ms,   OPT2, HIGH },	// Eigentlich werden fehlende Aktionen bei c1_H ggf. nachgezogen, aber bei der 80D gibt es dann wohl einen Konflikt mit Opt1
	{0 ms,   OPT2, LOW },
	LONGWAIT_ACTION,	// hier warten, bis Coil 1 Hi wird und die Belichtung beendet
#define c1_H_FinishShoot (c2_H_FinishShoot + 4)	
	// Coil 1 wird inaktiv: 
	// Curtain von Coil 2 hat schon vorher geöffnet und 2 ms danach ging OPT2 Lo - falls die Belichtung
	//  > 5 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt !
	{ 2 ms,  OPT1, LOW  },
	{ 3 ms,  OPT1, HIGH },
	{ 4 ms,  OPT1, LOW  },
	{ 7 ms,  M5 , HIGH }, 
	{ 20 ms, OPT1, HIGH },    // wg. Opt-Power=Off bei 15 ms ist der Zeitpunkt nicht zu erkennen,
	{ 20 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	{ 26 ms, ROT2, HIGH },
	{ 26 ms, ROT3, HIGH },
	//{30 ms,  M4 , LOW  },  // 26 ist zu kurz, 30 ok
	{40 ms,  M4 , LOW  },	// 45 geht auch
	{ 48 ms, ROT2, LOW  },
	{ 57 ms, ROT1, LOW  },
	STOP_ACTION,
};

/////////////////////////////////////
// Start nach Shoot im LV:  Coil 2 ohne Coil 1 aktiv
//
sAction c2_L_StartInLVForLV[] = {	
	{14 ms,  ROT1, HIGH },
	{16 ms,  ROT2, HIGH },
	{30 ms,  ROT1, LOW  },
	HALT_ACTION,		// weiter gehts mit C2 Hi
};


//
// Start in LV: Coil 1+2 aktiv. Wird für LV-Shoot und LV-Stop ausgef.
sAction c1_L_StartInLV[] = {			// Coil 2 aktiviert im LV
	{ 5 ms,  ROT1, HIGH },
	{14 ms,  ROT2, LOW  },
	{16 ms,  ROT3, LOW  }, 
	HALT_ACTION,
};

// Fortsetzung des LVShoot-Timings nach Coil-1/2 Hi
// !!! 6D: Klappt auch mit c2_H_FinishShoot: Spiegelsignale werden ignoriert. !!!
//
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
      //{4 ms,    OPT2, HIGH },	 OPT2 bleibt LOW!
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	{ 2 ms,  OPT1, LOW  },
	{ 3 ms,  OPT1, HIGH },
	{ 4 ms,  OPT1, LOW  },
	{15 ms,  OPT2, HIGH },  //  OPT2 wird hier oder nach Abschalten der OptPwr H
	{20 ms,  OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 22 ms, ROT2, HIGH },
	{ 22 ms, ROT3, HIGH },
	{ 44 ms, ROT2, LOW  },
	{ 53 ms, ROT1, LOW  },
	STOP_ACTION,
};


// Beim Start des LV löst Coil 2 erst nach 64 ms (ohne Coil-1 Signal).
// Wenn anhand der C2_L Dauer vom normalen Shoot unterschieden wird, sind die ROT-Signale
//  schon zu weit, weil der Motor im LV-Leise-1 das Schließen noch nicht vorspannt.
//  Das scheint zunächst kein Problem zu sein, aber beim Shoot im oder Stop des LV 
//  gibts Error 20. Die Pegel hier zu korrigieren geht aber nicht!
sAction c2_H_EnterLV[] = {	
	{2 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 2 ms geht OPT2 Lo. 
	{3 ms,   OPT2, HIGH },	
	{5 ms,   OPT2, LOW },	// das ist nicht zu sehen, weil OptPwr abgesch. wird
	STOP_ACTION,
};
#define c1_H_EnterLV c2_H_EnterLV

// Nachdem das LV-Ende am Coil-2 Hi Timing erkannt wurde, müssen/können(?) 
//  die ROT-Pegel noch korrigiert werden, die hier früher kommen müssen,
//  aber noch auf dem Stand der reg. Auslösung nach 28 ms sind:
//
sAction c2_H_LeaveLV[] = {
	{ 2 ms,  OPT1, LOW  },
	{ 3 ms,  OPT1, HIGH },
	{ 4 ms,  OPT1, LOW  },
	{ 7 ms,  M5 , HIGH }, 
	{62 ms,  M4 , LOW  },	// nach diesem Signal kommt Err20. Je mehr es verzögert wird, desto länger ist die normale Anzeige sichtbar.
	{77 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{77 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	{81 ms,  ROT2, HIGH },
	{81 ms,	 ROT3, HIGH },
	{102 ms, ROT2, LOW  },
	{111 ms, ROT1, LOW  },
//{355 ms,  M4 , LOW  },	// nach diesem Signal kommt Err20
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// ohne Coil2: für Coil1 identisch

///////////////////////////////////////

/*  80D Coil-Timing (LV im Leise-Modus 1):
	Modus     Coil 1  Coil 2
	------    ------  ------
	Shoot      47+t     47
	Shoot LV   54+t     54
	Enter LV    --      64	  // C1 wird für (Re-)EnterLV nicht aktiviert. Da C2 immer zeitgleich
	ReEnter LV  --      38	  //  mit C1 startet, wird C2_H als Basissignal verwendet!
	Leave LV    21      21

  Die Zeiten ändern sich, wenn der Spiegel ausgebaut oder nicht eingehängt ist:
    Enter_LV kommt dann z.B. schon nach 52 atstt 64 ms!
  In der Voll-Emulation verschiebt sich das Ende von c2_L_StartForLV mit MR2 Low: offenbar wartet die Kamera auf den Spiegel.
    C2_H nach c1_L_Start kommt erst nach 108 statt 47 ms - obwohl MR2_L schon nach 18 ms angelegt wird !?!
	 
*/
// --> Grenzwerte zur Unterscheidung der Funktion bei C2_H:
#define HiLIMIT_C2_LVSHOOT	((54+5) ms)
#define HiLIMIT_C2_SHOOT	((47+5) ms)
#define HiLIMIT_C2_REENTER_LV	((38+6) ms)
#define LoLIMIT_C2_ENTER_LV	((64-4) ms)
#define HiLIMIT_C2_LEAVE_LV	((21+6) ms)

#define IsInWindow_EnterLV(dt)   (dt >= HiLIMIT_C2_LVSHOOT)
//#define IsInWindow_LVShoot(dt)   (dt <  HiLIMIT_C2_LVSHOOT  &&  dt >= HiLIMIT_C2_REENTER_LV)
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C2_REENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_LeaveLV(dt)   (dt <  HiLIMIT_C2_LEAVE_LV)

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
print_us(dt);
  if (actionPtr == c2_L_StartForLV) {	// über Timing klappts nicht, weil C2 in der Emulation nur 51..52ms L ist (statt 64)!!!
    PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
//#ifdef EOS80D_VOLLEMU	    // auch Spiegel wird emuliert: Timing verändert! Deshalb:
  if (actionPtr == c1_L_Start) {	// über Timing klappts nicht, weil C2 in der Emulation nur 51..52ms L ist (statt 64)!!!
    PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;
  }
//#endif
  if (dt <  HiLIMIT_C2_LEAVE_LV) {	// LV-Stop: dT < 21+6 ms
    PRINTLN("c2_H_LeaveLV");
    return c2_H_LeaveLV;
  }
  else if (dt <= HiLIMIT_C2_REENTER_LV) { // ReEnter LV: ~38
    PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
  else if (dt < HiLIMIT_C2_SHOOT) {	// vermutlich zu kurz für große Blendenwerte "80D_BLENDE"
					// evtl. etwas erhöhen und sicherstellen, dass nicht im LV
					// (wg. Konflikt mit HiLIMIT_C2_LVSHOOT) und < 
    // --> testen:
//else if (dt < LoLIMIT_C2_ENTER_LV  &&  actionPtr != c1_L_StartInLV) {
    PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;		// Shoot (47ms)
  }
  else if (dt <  HiLIMIT_C2_LVSHOOT) {	// Auslösen im LV: (38+6) ms < dt <  ((54+5) ms)
    PRINTLN("c2_H_FinishLVShoot");
    return c2_H_FinishLVShoot;
  }
  else { //if (dt >= HiLIMIT_C2_LVSHOOT) 
    PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
/*
  if (IsInWindow_LeaveLV(dt)) {		// LV-Stop: dT < 21+6 ms
    PRINTLN("c2_H_LeaveLV");
    return c2_H_LeaveLV;
  }
  else if (IsInWindow_LVShoot(dt)) {	// Auslösen im LV: (38+6) ms < dt <  ((54+5) ms)
    PRINTLN("c2_H_FinishLVShoot");
    return c2_H_FinishLVShoot;
  }
  else if (IsInWindow_EnterLV(dt) || IsInWindow_ReEnterLV(dt)) {			
//BlinkLED(31 ms);
    PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;		// Shoot (52ms)
  }
*/
}

// Erste sAction Liste auswählen für ein Coil-Start Ereignis
//
inline sAction* SelectStartAction()
{
  if (digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {   // auch Coil 1 ist aktiv: Auslösen oder LV-Stop
    if (actionPtr == c2_H_EnterLV) {	  // zuletzt in den LV gegangen?
      PRINTLN("c1_L_StartInLV");
      return c1_L_StartInLV;
    }
    else {
      PRINTLN("c1_L_Start");
      //BlinkLED(14 ms);
      return c1_L_Start;
    }
  }
  else {		  // ohne Coil 1: LV-Start - ist innerhalb nach LVShoot oder LV-ReEnter möglich
    if (actionPtr == c2_H_EnterLV  ||  actionPtr == c2_H_FinishLVShoot) {		  
      PRINTLN("c2_L_StartInLVForLV");
      return c2_L_StartInLVForLV;
    }
    else {
      PRINTLN("c2_L_StartForLV");
      return c2_L_StartForLV;
    }
  }
}

#endif // EOS80D --------------------------------------------

#ifdef EOS70D
/*
Der Shutter der 70D ist nahezu baugleich mit dem der 80D,
 wird aber ähnlich wie der der 60D betrieben:
 Coil 1 und 2 werden immer gemeinsam aktiviert, Coil 1 beendet
   die Belichtung.
 Die ROT-Signale zeigen gewisse Unterschiede (ROT1/3 Bezeichnung ist vertauscht!)
 Die Coil 2 Dauer zeigt die aktuelle Aktion:
  49 ms: Shoot
  53 ms: Shoot im LV (Leise 1)
  39 ms: LV Start (Leise 1)
  17 ms: LV Stopp (Leise 1)
*/

// Start einer Aktion außerhalb des LV
//
sAction c1_L_Start[] = {			// Coil 2 aktiv
/*	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, HIGH },
	{0 ms,   ROT1, LOW  },
	{0 ms,   ROT2, LOW  },
	{0 ms,   ROT3, HIGH },	// Ausgangszustand */
	{ 1 ms,  MR2 , HIGH },
	{ 4 ms,  MR3 , LOW  },
	{ 6 ms,  ROT1, HIGH },
	{ 7 ms,  ROT2, HIGH },
	{ 9 ms,  MR3 , HIGH },
	{10 ms,  MR1 , LOW  },
	{12 ms,  ROT1, LOW  },
	//
	// In diesem Zustand kommt bei LV1-Start das C2_H bei 39 ms.
	// Für normale Auslösung (C2_H bei 49 ms) müssen die nächsten 
	//  Flanken auf jeden Fall schon ausgeführt werden.
	/*
	{16 ms,  ROT1, HIGH },  // (das erste würde beim LV-Start nicht stören)
	{25 ms,  ROT2, LOW  },
	{27 ms,  ROT3, LOW  },
	/*/
	// Die nächsten 3 Aktionen sind hinter 39 ms versetzt, um zu vermeiden,
	//  dass sie beim LV-Start, wo sie erst beim nächsten Ereignis folgen, ausgeführt werden.
	//  Die Kamera wartet später endlos darauf, wogegen auch keine Korrektur in EnterLV hilft.
	// Für die normale Auslösung ist das kein generelles Problem, weil die
	//  Kamera mit C2_H wartet. Mit 41 ms für die nächste Aktion kommt das C2_H noch bei 49 ms.
	// (Zur besseren Unterscheidung von anderen C2_H könnte auch noch weiter verzögert
	//  werden, z.B. um C2_
	// 
	{41 ms,  ROT1, HIGH },
	{41 ms,  ROT2, LOW  },
	{42 ms,  ROT3, LOW  },
	//*/
	HALT_ACTION,		// weiter gehts wenn C2 Hi die Belichtung startet
};


// Beim Start des LV löst Coil 2 schon nach 39 ms.
// Wenn anhand der C2_L Dauer vom normalen Shoot unterschieden wird, sind die ROT-Signale
//  schon zu weit, weil der Motor im LV-Leise-1 das Schließen noch nicht vorspannt.
//  Bei der 60D scheint das zunächst kein Problem zu sein, aber beim Shoot im oder Stop des LV 
//  gibts Error 20. Die Pegel hier zu korrigieren geht aber nicht!
//  Deshalb werden die ROT2+3 LOW Flanken in c1_L_Start verzögert
//  (ROT1 HIGH auch, könnte aber schon ausgeführt werden).
//
sAction c2_H_EnterLV[] = {	
	//{0 ms,   ROT2, HIGH}, // Korrigieren der Flanken, die in c1_L_Start schon
	//{0 ms,   ROT3, HIGH}, //   gesetzt wurden HILFT NICHT! (s. c1_L_Start)
	//{0 ms,   ROT1, LOW },
	{3 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 4 ms geht OPT2 Lo. Bei sehr kurzen Belichtungen (<6ms, 1/200) wird die Aktion vorgezogen.
	{4 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{5 ms,   OPT2, LOW },
	STOP_ACTION,
};

sAction c1_H_EnterLV[] = {	// Coil 1 wird zwar geschaltet, aber der Curtain ist nicht aufgezogen und schließt nicht
	STOP_ACTION,
};


//
// Fortsetzung des regulären Shoot-Timings nach Coil-2 Hi und später C1_H
//
sAction c2_H_FinishShoot[] = {		// Coil 2 wird inaktiv
	{3 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 2-3 ms geht OPT2 Lo.
	{4 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{5 ms,   OPT2, LOW },
	LONGWAIT_ACTION,	// hier warten, bis Coil 1 Hi wird und die Belichtung beendet
#define c1_H_FinishShoot (c2_H_FinishShoot + 4)	
	// Coil 1 wird inaktiv: 
	// Curtain von Coil 2 hat schon vorher geöffnet und 4 ms danach ging OPT2 Lo - falls die Belichtung
	//  > 5 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt und deshalb hier ggf. nachgezogen! (Perfekt wäre Multi-Action!)
	// Klappt auch mit 1/2000, obwohl die Flanken dann sehr schnell kommen.
	// Vmtl. ist es auch ok, OPT2_H sofort nach C2_H auszuführen.
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  MR1 , HIGH }, 
	{30 ms,  MR3 , LOW  }, 
	{ 33 ms, OPT1, HIGH },    // wg. Opt-Power=Off bei 15 ms ist der Zeitpunkt nicht zu erkennen,
	{ 33 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	{ 37 ms, ROT3, HIGH },
	{ 37 ms, ROT2, HIGH },
	{37 ms,  MR3 , HIGH }, 
	{38 ms,  MR2 , LOW  }, 
	{ 58 ms, ROT2, LOW  },
	{ 64 ms, ROT1, LOW  },
	STOP_ACTION,
};


// Im LV1 wird die ShootLV-Sequenz direkt ausgeführt:
//
sAction c1_L_StartInLV[] = {	    // Timing für Shoot im LV1	
//*
	{0 ms,   OPT1, HIGH },
	{0 ms,   ROT3, HIGH  },
	{0 ms,   ROT2, HIGH  },
	{0 ms,   ROT2, HIGH  },
	{0 ms,   ROT1, LOW  },	// soweit unnötig */
	{ 5 ms,  ROT1, HIGH },// GERATEN!
	{14 ms,  ROT2, LOW  },
	{16 ms,  ROT3, LOW  },
	HALT_ACTION,		// weiter gehts mit C1 Hi sehr ähnlich wie Shoot ohne LV
};

/* Test Leise1_1:
#define c1_H_FinishLVShoot c1_H_FinishShoot 
#define c2_H_FinishLVShoot c2_H_FinishShoot 
/*/
// Fortsetzung des LVShoot-Timings nach Coil-1/2 Hi
// !!! 60D: Klappt auch mit c2_H_FinishShoot: Spiegelsignale werden ignoriert. !!!
//
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
      //{4 ms,    OPT2, HIGH },	// -> Curtain öffnet (OPT2 war gar nicht LOW)
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 22 ms, ROT3, HIGH },
	{ 23 ms, ROT2, HIGH },
	{ 43 ms, ROT2, LOW  },
	{ 54 ms, ROT1, LOW  },
	{ 54 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 54 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	STOP_ACTION,
};
//*/

// Das LV-Ende wurde am Coil-1 oder -2 Hi Timing erkannt. Dann folgt:
//
sAction c2_H_LeaveLV[] = {
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 5 ms,  ROT1, HIGH },
	{ 7 ms,  MR1 , HIGH },
	{ 14 ms, ROT2, LOW  },
	{ 16 ms, ROT3, LOW  },
	{ 29 ms, MR3 , LOW  },
	{ 37 ms, MR3 , HIGH },
	{ 38 ms, MR2 , LOW  },
	{ 36 ms, ROT2, HIGH },
	{ 36 ms, ROT3, HIGH },
	{ 56 ms, ROT2, LOW  },
	{ 58 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 58 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	{ 64 ms, ROT1, LOW  },
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// mit oder ohne Coil2: für Coil1 identisch

///////////////////////////////////////
/*  70D Coil-Pulsdauer (LV im Leise-Modus 1):
	Modus	   Coil 1  Coil 2

	Shoot      49+t     49	  // wird durch die zurückgestellte ROT-Signale ggf. verzögert!	 
	Shoot LV   53+t     53
	Enter LV    44      39
	ReEnter LV  35      30
	Leave LV    17      17

	Die Zeiten können durch Verschieben der ROT-Signale verändert/angepasst werden! 
*/

// --> Grenzwerte zur Unterscheidung der Funktion bei C1_H:
#define HiLIMIT_C2_ENTER_LV	((39+4) ms)
#define HiLIMIT_C2_REENTER_LV	((30+4) ms)
#define HiLIMIT_C2_LEAVE_LV	((17+6) ms)
#define LoLIMIT_C2_LVSHOOT	((53-2) ms)

//#define IsInWindow_LVShoot(dt)   (dt >= LoLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C2_ENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C2_REENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C2_LEAVE_LV)

//
// Erste sAction Liste auswählen für ein Coil-Start Ereignis
// In allen Modi wird nach Shoot im LV wieder ein EnterLV ausgeführt, so dass
//  daran immer zu erkennen ist, ob LV aktiv ist.
//
// Nicht verwendet (s. Loop()):
inline sAction* SelectStartAction()
{
  if (actionPtr == c2_H_EnterLV) {	// im LV?
PRINTLN("c1_L_StartInLV");
    return c1_L_StartInLV;
  }
  else {
PRINTLN("c1_L_Start");
    return c1_L_Start;
  }
}
/*
C1_L c1_L_StartInLV
11:39:08.368 -> C2_H 31ms c2_H_EnterLV
11:39:08.368 -> C2_H 21ms c2_H_EnterLV
11:39:08.436 -> C1_H C1_L c1_L_StartInLV
11:39:09.248 -> C1_H c1_H_FinishLVShoot
11:39:21.708 -> C2_H 12462ms c2_H_FinishLVShoot
*/

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
print_us(dt);
  if (actionPtr == c1_L_StartInLV  &&	// Re-EnterLV tritt in Emu auch mit c1L=22ms auf! Deshalb hier Leave nur zulassen, wenn im LV!
      dt <= HiLIMIT_C2_LEAVE_LV) {		// LV-Stop (17ms)
PRINTLN("c2_H_LeaveLV");
    return c2_H_LeaveLV;
  }
  else if (dt < HiLIMIT_C2_ENTER_LV) {		// (Re-)EnterLV (38 ms)
PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
  else if (dt > LoLIMIT_C2_LVSHOOT) {		// Auslösen im LV (58ms)
PRINTLN("c2_H_FinishLVShoot");
    return c2_H_FinishLVShoot;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;			// Shoot (49++ ms, verzögert)
  }
}

#endif // EOS70D --------------------------------------------

#ifdef EOS60D

sAction c1_L_Start[] = {			// Coil 2 aktiv
	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, HIGH },
	{0 ms,   ROT2, LOW  },
	{0 ms,   ROT3, LOW  },
	{0 ms,   ROT1, HIGH },	// soweit unnötig
	{ 2 ms,  S4  , HIGH },
	{ 8 ms,  S5  , LOW  },
	{11 ms,  ROT2, HIGH },
	{11 ms,  ROT3, HIGH },
	{15 ms,  S6  , LOW  },
	{18 ms,  ROT3, LOW  },
	// In diesem Zustand kommt bei LV1-Start C2_H, normal erst nach 38 ms.
	// Zumindest die nächste Flanke wird in der Emu. des LV-Start aber auf jeden
	//  Fall schon ausgeführt, was die Kamera aber nicht stört.
	{23 ms,  ROT3, HIGH },
	// Die beiden folgenden Aktionen sind um 10 ms nach hinten versetzt, um zu vermeiden,
	//  dass sie beim LV-Start, wo sie erst beim nächsten Ereignis folgen, ausgeführt werden.
	//  Für die normale Auslösung scheint das kein Problem zu sein, aber
	//  wenn sie beim LV-Start ausgeführt werden, wartet die Kamera später endlos darauf, 
	//  wogegen auch keine Korrektur in EnterLV hilft.
	{43 ms,  ROT2, LOW  },
	{46 ms,  ROT1, LOW  },
	HALT_ACTION,		// weiter gehts, wenn C2 Hi nach 55 ms die Belichtung startet (bei der Spiegelvorauslösung hält der Ablauf hier an)
};

//
// Fortsetzung des regulären Shoot-Timings nach Coil-2 Hi und später C1_H
//
sAction c2_H_FinishShoot[] = {		// Coil 2 wird inaktiv
	{3 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 2-3 ms geht OPT2 Lo.
	{4 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{5 ms,   OPT2, LOW },
	LONGWAIT_ACTION,	// hier warten, bis Coil 1 Hi wird und die Belichtung beendet
#define c1_H_FinishShoot (c2_H_FinishShoot + 4)	
	// Coil 1 wird inaktiv: 
	// Curtain von Coil 2 hat schon vorher geöffnet und 4 ms danach ging OPT2 Lo - falls die Belichtung
	//  > 5 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgeführt und deshalb hier ggf. nachgezogen! (Perfekt wäre Multi-Action!)
	// Klappt auch mit 1/2000, obwohl die Flanke dann sehr schnell kommen.
	// Vmtl. ist es auch ok, OPT2_H sofort nach C2_H auszuführen.
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 6 ms,  S5  , HIGH }, 
	{ 6 ms,  S6  , HIGH }, 
	{ 33 ms, OPT1, HIGH },    // wg. Opt-Power=Off bei 15 ms ist der Zeitpunkt nicht zu erkennen,
	{ 33 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	{36 ms,  S6  , LOW  }, 
	{40 ms,  S6  , HIGH }, 
	{ 44 ms, ROT1, HIGH },
	{ 44 ms, ROT2, HIGH },
	{59 ms,  S5  , LOW  }, 
	{67 ms,  S5  , HIGH }, 
	{67 ms,  S4  , LOW  }, 
	{ 68 ms, ROT2, LOW  },
	{ 81 ms, ROT3, LOW  },
	STOP_ACTION,
};


// Im LV0 wird vor Shoot zuerst der LV beendet mit C2_H = 22 ms wie bei LV-Stop (Spiegel bleibt aber offen!).
//  Die ROT kommen aber 22 ms früher, weil der Motor nicht anhält. Ggf. muss dafür doch der LV-Zustand
//  geloggt werden und dazu noch der LeaveLV Zeitpunkt.
// Im LV1 wird die ShootLV-Sequenz direkt ausgeführt:
//
sAction c1_L_StartInLV[] = {	    // Timing für Shoot im LV1	
	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, LOW  },
	{0 ms,   ROT1, HIGH  },
	{0 ms,   ROT2, HIGH  },
	{0 ms,   ROT3, LOW  },	// soweit unnötig
	{ 5 ms,  ROT3, HIGH },
	{15 ms,  ROT2, LOW  },
	{19 ms,  ROT1, LOW  },
	HALT_ACTION,		// weiter gehts mit C1 Hi sehr ähnlich wie Shoot ohne LV
};

//* Test Leise1_1:
#define c1_H_FinishLVShoot c1_H_FinishShoot 
#define c2_H_FinishLVShoot c2_H_FinishShoot 
/*/
// Fortsetzung des LVShoot-Timings nach Coil-1/2 Hi
// !!! Klappt auch mit c2_H_FinishShoot: Spiegelsignale werden ignoriert. !!!
//
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
      //{4 ms,    OPT2, HIGH },	// -> Curtain öffnet (OPT2 war gar nicht LOW)
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 24 ms, ROT1, HIGH },
	{ 24 ms, ROT2, HIGH },
	{ 48 ms, ROT2, LOW  },
	{ 60 ms, ROT3, LOW  },
	{ 60 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 60 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	STOP_ACTION,
};
//*/

// Beim Start des LV löst Coil 2 schon nach 38 ms (ohne Coil-1 4 ms später).
// Wenn anhand der C2_L Dauer vom normalen Shoot unterschieden wird, sind die ROT-Signale
//  schon zu weit, weil der Motor im LV-Leise-1 das Schließen noch nicht vorspannt.
//  Bei der 6D scheint das zunächst kein Problem zu sein, aber beim Shoot im oder Stop des LV 
//  gibts Error 20. Die Pegel hier zu korrigieren geht aber nicht!
sAction c2_H_EnterLV[] = {	
	//{0 ms,    ROT2, HIGH}, // korrigieren: wurde in c1_L_Start schon gesetzt
	//{0 ms,    ROT1, HIGH}, //  HILFT ABER NICHT! --> c1_L_Start
	{3 ms,   OPT2, LOW },	// -> Curtain öffnet und nach 4 ms geht OPT2 Lo. Bei sehr kurzen Belichtungen (<6ms, 1/200) wird die Aktion vorgezogen.
	{4 ms,   OPT2, HIGH },	// Dann folgt noch ein kurzer Puls, der mit 1 ms Auflösung nicht immer zu sehen ist!!!!!!!
	{5 ms,   OPT2, LOW },
	STOP_ACTION,
};

sAction c1_H_EnterLV[] = {	// Coil 1 wird zwar geschaltet, aber der Curtain ist nicht aufgezogen und schließt nicht
	STOP_ACTION,
};


// Das LV-Ende wurde am Coil-1 oder -2 Hi Timing erkannt. Dann folgt:
//
sAction c2_H_LeaveLV[] = {
	{ 3 ms,  OPT1, LOW  },
	{ 4 ms,  OPT1, HIGH },
	{ 5 ms,  OPT1, LOW  },
	{ 6 ms,  S5  , HIGH },
	{ 6 ms,  S6  , HIGH },
	{37 ms,  S6  , LOW  },
	{41 ms,  S6  , HIGH },
	{ 44 ms, ROT1, HIGH },
	{ 44 ms, ROT2, HIGH },
	{61 ms,  S5  , LOW  },
	{ 68 ms, ROT2, LOW  },
	{69 ms,  S5  , HIGH },
	{69 ms,  S4  , LOW  },
	{ 82 ms, ROT3, LOW  },
	{ 82 ms, OPT1, HIGH },    // wg. Opt-Power=Off ist der Zeitpunkt nicht zu erkennen,
	{ 82 ms, OPT2, HIGH },	  //  aber die Curtains werden wieder in Startpos. gebracht!
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// mit oder ohne Coil2: für Coil1 identisch

///////////////////////////////////////
/*  60D Coil-Timing (LV im Leise-Modus 1):
	Modus	   Coil 1  Coil 2

	Shoot      55+t     55	  //bis über 58 in der Emu!)
	Shoot LV   58+t     58
	Enter LV    42      38
	ReEnter LV  38      34	  // 22 in der Emu!
	Leave LV    22      22

	Die Zeiten können im Emu sehr stark abweichen! 
*/

// --> Grenzwerte zur Unterscheidung der Funktion bei C1_H:
#define HiLIMIT_C2_ENTER_LV	((38+4) ms)
#define HiLIMIT_C2_REENTER_LV	((34+2) ms)
#define HiLIMIT_C2_LEAVE_LV	((22+6) ms)
#define LoLIMIT_C2_LVSHOOT	((58-0) ms)   // Shoot kann bis 57+ gehen

//#define IsInWindow_LVShoot(dt)   (dt >= LoLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C2_ENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C2_REENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C2_LEAVE_LV)

//
// Erste sAction Liste auswählen für ein Coil-Start Ereignis
// In allen Modi wird nach Shoot im LV wieder ein EnterLV ausgeführt, so dass
//  daran immer zu erkennen ist, ob LV aktiv ist.
//
// Nicht verwendet (s. Loop()):
inline sAction* SelectStartAction()
{
  if (actionPtr == c2_H_EnterLV) {	// im LV?
PRINTLN("c1_L_StartInLV");
    return c1_L_StartInLV;
  }
  else {
PRINTLN("c1_L_Start");
    return c1_L_Start;
  }
}

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
print_us(dt);
  if (actionPtr == c1_L_StartInLV  &&	// Re-EnterLV tritt in Emu auch mit c1L=22ms auf! Deshalb hier Leave nur zulassen, wenn im LV!
      dt <= HiLIMIT_C2_LEAVE_LV) {		// LV-Stop (22ms)
PRINTLN("c2_H_LeaveLV");
    return c2_H_LeaveLV;
  }
  else if (dt < HiLIMIT_C2_ENTER_LV) {		// (Re-)EnterLV (38 ms)
PRINTLN("c2_H_EnterLV");
    return c2_H_EnterLV;
  }
  else if (dt > LoLIMIT_C2_LVSHOOT) {		// Auslösen im LV (58ms)
PRINTLN("c2_H_FinishLVShoot");
    return c2_H_FinishLVShoot;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
PRINTLN("c2_H_FinishShoot");
    return c2_H_FinishShoot;			// Shoot (55ms)
  }
}

#endif // EOS60D --------------------------------------------

#ifdef EOS50D

sAction c1_L_Start[] = {		// Coil 1 (und 2) aktiv
	{0 ms,   OPT1, HIGH },
	{0 ms,   OPT2, HIGH },
	{0 ms,   ROT1, LOW  },
	{ 5 ms,  MR3 , HIGH  },
	{ 9 ms,  MR3 , HIGH },
	{12 ms,  MR1 , LOW  },
	{13 ms,  ROT1, HIGH },
	{33 ms,  ROT2, LOW  },
	// Hier folgt beim Shoot nach 50 bis 60 ms C2_H, das die Umschaltung auf c2_H_FinishShoot
	//  triggert. Beim Re-Enter des LV vergehen 240 ms für Zeiten < 1s.
	//  Für t > 1s und Bulb dauert es im LV1 und LV2 noch 860 ms, weil der Motor anhält: Dann
	//  sind die gerade ausgeführten ROT1_H und ROT2_L noch gar nicht fällig, sondern werden
	//  erst bei 514 und 535 ms erwarten. Deshalb nach 500  ms ROT1+2 zurücknehmen und 
	//  nochmal ausführen. Das Nachholen dieser Sequenz via HALT_ACTION scheint nicht zu stören.
	//  (Stattdessen könnte hier vmtl. auch STOP_ACTION verwendet werden.)
	{500 ms,  ROT1, LOW  },
	{500 ms,  ROT2, HIGH  },
	{514 ms,  ROT1, HIGH  },
	{535 ms,  ROT2, LOW  },
	HALT_ACTION,		// weiter gehts wenn C2 Hi die Belichtung startet (bei der Spiegelvorauslösung hält der Ablauf hier an)
};


// Fortsetzung des regulären Shoot-Timings nach Coil-2 Hi
//
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{ 3 ms,  OPT2, LOW  },
	{ 5 ms,  OPT2, HIGH },
	{ 6 ms,  OPT2, LOW  },
	LONGWAIT_ACTION,	// hier warten, bis Coil 1 Hi wird und die Belichtung beendet
#define c1_H_FinishShoot (c2_H_FinishShoot + 4)	
	// Coil 1 wird inaktiv: 
	{ 3 ms,  OPT1, LOW  },
	{ 5 ms,  OPT1, HIGH },	  // tritt evtl. nicht immer auf
	{ 6 ms,  OPT1, LOW  },	  //   "
	{10 ms,  MR1 , HIGH }, 
	{24 ms,  ROT2, HIGH },
	{91 ms,  MR3 , LOW  },
	{97 ms,  MR2 , LOW  },

	{46 ms,  OPT2, HIGH },
	{54 ms,  OPT1, HIGH },
	{56 ms,  OPT2, LOW  },
	{60 ms,  OPT2, HIGH },
	//Der folgende Puls ist nur im LV da, wird aber nicht gebraucht und auch im normalen Shoot ignoriert:
	//{66 ms,  OPT1, LOW  },
	//{77 ms,  OPT1, HIGH },
	{84 ms,  ROT1, LOW  },

	STOP_ACTION,
};
//
// Das einfache Shoot-Timing funktioniert auch für Start und Ende des LV und für
//  Shoot im LV ohne Leise-Modus. Für Shoot im Leise-Modus 1 und 2 für
//  Zeiten >= 1 Sekunde oder Bulb (egal wie lang) folgt Error 20.
//  Hier macht der Motor eine Pause, so dass Rot1 und Rot2 nicht schon
//   13 ms nach C1_H kommen, sondern erst nach 514 ms. Das ist im 
//   Shoot-Timing oben nachgebildet, so dass keine speziellen LV-Sequenzen
//   benötigt werden!
// Spiegelvorauslösung ist für normales Shoot ok, gibt bei Shoot im LV aber Err20.
//
/*
// Start nach Shoot im LV:  Coil 2 ohne Coil 1 aktiv
//
sAction c2_L_StartInLVForLV[] = {	
	HALT_ACTION,		// weiter gehts mit C2 Hi
};


//
// Start in LV: Coil 1+2 aktiv. Wird für LV-Shoot und LV-Stop ausgef.
sAction c1_L_StartInLV[] = {			// Coil 2 aktiviert im LV
	HALT_ACTION,
};

// Fortsetzung des LVShoot-Timings nach Coil-1/2 Hi
//
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	STOP_ACTION,
};

// Start des LV 
sAction c2_H_EnterLV[] = {	
	STOP_ACTION,
};
#define c1_H_EnterLV c2_H_EnterLV

sAction c2_H_LeaveLV[] = {
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// ohne Coil2: für Coil1 identisch
*/

#define	c1_H_FinishLVShoot  c1_H_FinishShoot
#define	c1_H_LeaveLV	    c1_H_FinishShoot
#define	c1_H_EnterLV	    c1_H_FinishShoot


#define IsInWindow_LVShoot(dt)   false	//(dt >= LoLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   false	//(dt <= HiLIMIT_C2_ENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_ReEnterLV(dt) false	//(dt <= HiLIMIT_C2_REENTER_LV  &&  dt >= HiLIMIT_C2_LEAVE_LV)
#define IsInWindow_LeaveLV(dt)   false	//(dt <= HiLIMIT_C1_LEAVE_LV)

// Erste sAction Liste auswählen für ein Coil-Start Ereignis
//
inline sAction* SelectStartAction()
{
  return c1_L_Start;
}

// Nächste sAction Liste auswählen für die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
  // Coil-2 Timing der 50D:
  // - Das reguläre C2_Hi kommt bei 53-54 ms, im LV große Unterschiede je nach Modus.
  // - Beim LV-Start kommt C2_Hi nach 51 ms oder 54 ms (LV1).
  // - Am Ende des LV ist der C2 Puls ca. 240 ms breit
  //
  return c2_H_FinishShoot;
}

#endif // EOS50D --------------------------------------------

int stateIndex;      				// Status im Ablauf
unsigned long timingZero;   // Start-Zeit

inline void SetPin(int pin, int val) __attribute__((always_inline));
inline unsigned long timeElapsed() __attribute__((always_inline));
#define state_Off -1


void loop()
{
  //
  // Ablaufschema:
  // ------------------------------------
  // Vor einer Auslösung oder bei länger geöffnetem Shutter
  //  befindet sich der Prozessor im Standby, aus der ihn eine Flanke von Coil 1 oder
  //  ggf. Coil 2 weckt und in triggerEvent einen Code einträgt.
  //
  // Der Trigger weckt auch den Prozessor, so dass loop() wieder durchlaufen wird,
  //  wo dann die zu triggerEvent passende Aktionsliste gestartet wird.
  //
  // Bis zum Ende der aktuellen Pulsgruppe wird loop() durchlaufen und geprüft,
  //  ob die in actionPtr[stateIndex].time vermerkte Zeit erreicht ist. 
  //  Wenn ja, wird die hinterlegte Aktion ausgeführt und stateIndex erhöht.
  //  LONGWAIT_VAL und HALT_VAL Einträge halten die Ausführung bis zum nächsten
  //  Trigger an, STOP_VAL beendet die laufende Aktionsliste.
  //
  // Ablauf eines Ereignisses mit Coil 1 (6D, 80D: Coil 2) als Trigger:
  // ------------------------------------------------------------
  // 1.) Zuerst wird geprüft, ob Coil 1 eine Änderung erfahren hat (triggerEvent wurde im Int-Handler gesetzt).
  //  a)  1 -> 0   stateIndex wird auf 0 gesetzt, timingZero erhält die aktuelle Zeit.
  //               (Viele Modelle schalten regelmäßig, andere nur bei Half-Shoot, die Coil-Pwr aktiv.
  //		    Dadurch erzeugen reale Coils Signale, die aber mit einem Emulator nicht auftreten, weil die Coils
  //                gegen einen konstanten Pullup arbeitet, die emulierte Coil-Pwr also immer anliegt!).
  //  b)  0 -> 1   Zweiter Teil (Schließen) beginnt, stateIndex wird bis hinter einen eventuellen LONGWAIT
  //               inkrementiert. Noch anstehende Aktionen werden dabei ausgeführt.
  //               Die positive Coil-Flanke setzt auch die Zeitbasis für den Rest der Signale neu.
  //
  // Mögliche Folgen mit Coil 1 allein:
  //  c1_L_Start - c1_H_FinishShoot
  //  c1_L_Start - c1_H_EnterLV
  //  c1_L_StartInLV - c1_H_FinishLVShoot  +  c1_L_StartInLV - c1_H_EnterLV
  //  c1_L_StartInLV - c1_H_LeaveLV
  //
  // Mögliche Folgen mit Coil 2 (pos. Flanke) allein:
  //  c1_L_Start - c2_H_FinishShoot (mit LONGWAIT bis c1_H)
  //  c1_L_Start - c2_H_EnterLV
  //  c1_L_StartInLV - c2_H_FinishLVShoot (mit LONGWAIT bis c1_H) +  c1_L_StartInLV - c2_H_EnterLV
  //  c1_L_StartInLV - c2_H_LeaveLV  (nicht für 600D: da bleibt C2 inaktiv
  //
  // Der Vorteil von c2_H liegt darin, dass es unabhängig von der Belichtungszeit zu
  //  einem festen Zeitpunkt kommt. Wahlweise können die c2_H Timingtabellen, die auf
  //  c1_H warten müssen, ein LONGWAIT enthalten, oder das c1_H Event wählt eine neue
  //	Tabelle.
  //
  if (triggerEvent == evCoil1Start) {        // Start-Flanke (Coil 1 - oder, wenn 6D, Coil 2 - aktiviert):
    prevEvent = triggerEvent;
    triggerEvent = evNone;
    timingZero = triggerTime;		      //    Zeitpunkt festhalten
    stateIndex = 0;
#if defined(EOS6D)  ||  defined(EOS80D)  ||  defined(EOS50D)	
    actionPtr = SelectStartAction();
#else
    //
    // Außerhalb des LV kann:
    //  a) eine normale Auslösung vorliegen, die mit c1_L_Start behandelt wird, oder
    //  b) ein Start des LV, für den c1_L_Start die ROT-Signale zu weit ausführt.
    //  c1_L_Start stoppt nach 48 ms und wartet auf Coil 2 --> Hi. Der C2_H Zeitpunkt
    //   lässt zwischen a) und b) unterscheiden.
    //
    // Im LV war die letzte Aktion entweder
    //  - coilX_FinishLVShoot, dem wieder ein cX_H_EnterLV folgen muss, oder
    //  - cX_H_EnterLV. Danach kommt entweder
    //  	a) LV-Auslösen oder 
    //  	b) LV-Ende. Dafür ist c1_L_Start ungeeignet, deshalb c1_L_StartInLV verwenden:
    //
    if (actionPtr == c1_H_EnterLV	// c1_H_EnterLV kann auch im Coil2-Modus vorkommen, wenn die nicht angeschlossen ist
#if defined(TRIGGER2_MODE)
      || actionPtr == c2_H_EnterLV
#endif
    // Achtung, für Coil 1 müsste hier eigentlich kommen:				??600D
    //|| actionPtr == c1_H_FinishLVShoot
    // aber	das klappt für die 600D nicht (Err20)!!
    // Ohne diese Prüfung wird nach LV-Shoot das jetzt folgende Re-Enter-LV mit c1_L_Start 
    //  statt mit c1_L_StartInLV (erfolgreich) eingeleitet. Dadurch wird dann bei C1_H das
    //  erste c1_H_EnterLV anstatt dem zweiten (dem Re-EnterLV) ausgewählt. Die sind eh gleich.
    // Der Fehler tritt auch auf, wenn c1_L_StartInLV gleich ist wie c1_L_Start. Er besteht
    //  vmtl. darin, dass nach c1_H_FinishLVShoot das C1_H Timing ncht richtig erkannt wird.
    // besser so Cam-spezifisch def.:
    //#define IS_IN_LV 		(actionPtr == cX_H_EnterLV  ||  actionPtr == c1_H_FinishLVShoot)
    // 
    ) {
      actionPtr = c1_L_StartInLV;
PRINTLN("c1_L_StartInLV");
    }
    else {
      actionPtr = c1_L_Start;
PRINTLN("c1_L_Start");
    }
#endif
  } // if (triggerEvent == evCoil1Start) -----------------------------------------------
  
  else if (triggerEvent == evCoil1Stop) { // Coil 1 wurde inaktiv (C1_H):
    //
    // Falls eine Belichtung stattfand, sollte aktuell ein bei C2_H gestartetes LONGWAIT 
    //  laufen (entweder in c2_H_FinishShoot oder in c2_H_FinishLVShoot).
    // Das wird jetzt beendet (und die davor noch anstehenden Signale
    //  werden generiert).
    //
#if defined(TRIGGER2_MODE)	// ############# C1_H nach C2_H: Belichtung beendet
    if (prevEvent == evCoil2Stop) {
      if (stateIndex >= 0) {
 #if defined(EOS600D)
	//
	// die 600D generiert bei LeaveLV keinen C2 Puls. Deshalb via C1_H erkennen:
	//
	if (actionPtr == c1_L_StartInLV) {		// kein C2_H (und im LV)?
	  actionPtr = c1_H_LeaveLV;
	  stateIndex = 0;
	}
	else			// die anderen Möglichkeiten wurden schon via C2_H unterschieden!
 #endif //defined(EOS600D)
	do {							// hinter Wartepunkt in der c2_H-Liste gehen:
PRINT(".");
	  int v = actionPtr[stateIndex].value;
	  if (v == LONGWAIT_VAL) {				// WAIT beenden und weiter
	    stateIndex++;
PRINTLN("*+t");
	    break;
	  }
	  else if (v == STOP_VAL || v == HALT_VAL) {	// anhalten mit/ohne in Sleep zu gehen: erledigt
	    break;
	  }
	  else {					// noch nicht beim WAIT angelangt? (sollte nicht passieren)
	    SetPin(actionPtr[stateIndex].pin, v);	//  bis zum WAIT abarbeiten
	    stateIndex++;
	  }
	} while (1);
      }
    }
    else	// falls Coil2 nicht angeschlossen ist, kam auch kein C2_H:
#endif //defined(TRIGGER2_MODE)
    {
/*
if (actionPtr == c1_L_Start) {PRINTLN("c1_L_Start->");}
else if (actionPtr == c1_H_EnterLV) {PRINTLN("c1_H_EnterLV->");}
else if (actionPtr == c1_H_LeaveLV) {PRINTLN("c1_H_LeaveLV->");}
else if (actionPtr == c1_H_FinishShoot) {PRINTLN("c1_H_FinishShoot->");}
else if (actionPtr == c1_H_FinishLVShoot) {PRINTLN("c1_H_FinishLVShoot->");}
else {PRINTLN("???->");}
//*/
      unsigned long dt = triggerTime - timingZero;		// Coil-1 Pulsdauer
      if (actionPtr == c1_L_Start) {		// (noch) nicht im LV?
	if (IsInWindow_EnterLV(dt)) {
	  actionPtr = c1_H_EnterLV;		// dt passt zu LV-Start oder Restart -- FALSCH bei best. Belichtungszeiten (600D: < 1/60s!!!)
	  PRINTLN("c1_H_EnterLV");
	}
	else {
	  actionPtr = c1_H_FinishShoot;	// zu kurz/lang für EnterLV: reguläres (nicht zu kurzes!) Auslösen
	  PRINTLN("c1_H_FinishShoot");
	}
      }
      else { // im LV:
	      // a) Shoot im LV endet
	      // b) Re-Enter LV endet (folgt immer nach a)
	      // c) Leave LV endet: Kann nur nach EnterLV kommen, dt ist sehr kurz
	      //
	if (IsInWindow_LeaveLV(dt)) {		// LeaveLV hat kürzesten Puls, also zuverlässig zu erkennen!
	  PRINTLN("c1_H_LeaveLV");
	  actionPtr = c1_H_LeaveLV;
	}
	else if (IsInWindow_ReEnterLV(dt)
	  #ifdef EOS600D
		 &&  actionPtr != c1_L_StartInLV  // für 600D zusätzlich prüfen: kurze Zeiten im LV liegen unter HiLIMIT_C1_LEAVE_LV
	  #endif	  
	) {
	  PRINTLN("c1_H_(Re)EnterLV");
	  actionPtr = c1_H_EnterLV;
	}
	else {
	  // zu lange für EnterLV: reguläres (nicht zu kurzes!) Auslösen
	  // geht für 600D schief für kurze Belichtungen. Sicher zu unterscheiden, wenn vorheriger actionPtr gespeichert wird: Der war dann nicht
	  //  c1_H_FinishLVShoot, weil danach immer c1_H_EnterLV folgt
	  PRINTLN("c1_H_FinishLVShoot");
	  actionPtr = c1_H_FinishLVShoot;
	}
      }
      stateIndex = 0;
    }
    timingZero = triggerTime;							// Timing nach dem WAIT ab jetzt
    prevEvent = triggerEvent;
    triggerEvent = evNone;
  } // if (triggerEvent == evCoil1Stop) -----------------------------------------------------
  //
  //
#if defined(TRIGGER2_MODE)		// Auch Coil 2 Signale
  else if (triggerEvent == evCoil2Stop) {        // Coil-2 deaktiviert: Shutter öffnet
    unsigned long dt = triggerTime - timingZero;		// Coil-2 Pulsdauer
    timingZero = triggerTime;				      // neuer Startpunkt
    stateIndex = 0;
    actionPtr = SelectFinishAction(dt);
    prevEvent = triggerEvent;
    triggerEvent = evNone;
  } // if (triggerEvent == evCoil2Stop) ----------------------------------------
# endif	
  // Prüfen, ob das zum aktuellen Zustand gehörige Delay erreicht ist.
  //
  int v;
  if (stateIndex < 0) {
    v = STOP_VAL;
  }
  else {
    v = actionPtr[stateIndex].value;
    if (v == HIGH || v == LOW) {
      if (actionPtr[stateIndex].time <= timeElapsed()) {
	//
	// Status[stateIndex] ausgeben.
	// stateIndex weiter und ggf. auf state_off oder state_LongWait setzen.
	//
	SetPin(actionPtr[stateIndex].pin, v);
	stateIndex++;
	v = actionPtr[stateIndex].value;
	if (v == STOP_VAL) {
	  //BlinkLED(actionPtr[stateIndex-1].time);
	  stateIndex = state_Off;							// bereit für Sleep
	}
	//if (v == HALT_VAL) BlinkLED(3 ms);
      }
    }
  }
  //
  if (v == HALT_VAL) DoBlinkLED();
  //
  // Wenn die Pause oder das Ende erreicht werden, den Prozessor schlafen legen,
  //  andernfalls in der Loop() weitermachen:
  //
//ToggleDebugLED();
  if (stateIndex == state_Off || v == LONGWAIT_VAL) {
    if (DoBlinkLED() && triggerEvent == evNone) {
#if defined(EOS80D) && defined(MOTORDRV)
      if (stateIndex == state_Off) {
	// Emulation des Spiegelsignals M4, das schon ca. 2 ms vor Coil 2 LOW wird.
	// Die Versorgung für die Spiegel-Lichtschranken liegt schon ca. 5 ms vor
	//  den Coils an und wird hier genutzt, das M4_H rechtzeitig zu erzeugen.
	//  Behebt aber die Probleme nicht: Scheinbar ist M4_H unkritisch!
	if (digitalRead(MOTORDRV) == LOW) {
	  if (stime != 0) {
	    stime = 0;
//BlinkLED(6 ms);
	  }
	}
	else {
	  if (stime == 0) {
	    stime = micros();
	  }
	  else {
	    if (micros() - stime > (2 ms)) {
	      SetPin(M4, HIGH);
BlinkLED(30 ms);
	    }
	  }
	}
/*
#if defined(EOS80D) && defined(MOTORDRV)
  static unsigned long tt = 0;
  if (micros() - tt > (1 ms)) {
    tt = micros();
    SetPin(M4, !digitalRead(M4));
  }
#endif      
*/
/*
	//static unsigned long tt = 0;
	//if (micros() - tt > (1 ms)) {
	  //tt = micros();
	if (stime != 0  &&  (micros() - stime) > 2000 ms) {
	  stime = 0;
	  PRINTLN("MotorDrv");
	  SetPin(M4, HIGH);
	  //SetPin(M4, !digitalRead(M4));
	  //SetPin(M5, !digitalRead(M5));
	}
*/
      }
#elif !defined(SERIAL_DEBUG)	// sleep verhindert serielle Ausgabe!
      // Im SLEEP_MODE_STANDBY sind alle Timer aus! Deshalb wird HALT_VAL verwendet,
      //  um auf die Flanke der Coil 2 zu warten, weil für die das Timing benötigt wird.
      // Mit dem SLEEP_MODE_IDLE, der etwa 350 µA braucht, könnte auch diese Wartezeit
      //  mit sleep verbracht werden, aber das lohnt absolut nicht.
      //set_sleep_mode(SLEEP_MODE_PWR_DOWN); // den tiefsten Schlaf auswählen PWR_DOWN
      set_sleep_mode(SLEEP_MODE_STANDBY); // nach 6 Takten wieder da, 35 µA, Timer aus!

      sleep_enable(); // sleep mode einschalten
      sleep_mode(); // in den sleep mode gehen
      sleep_disable(); // sleep mode ausschalten nach dem Erwachen

      // --> "C:\User\Astro\Basteln\Arduino\Arduino schlafen legen.htm"
#endif 
    }
  }
}

//
// Explizit prüfen, ob die Cam gerade die Signale abgeschaltet hat.
// Nur sinnvoll, wenn ein Shutter eingebaut ist, der die Coil-Signale
//  niederohmig belastet. Wird aktuell für EOS6Dii_MIRROR benötigt.
// 
// Klappt (für die 6Dii) nicht zuverlässig:
// - ca. 1s nach LeaveLV kommt ein C2_L, das durchkommt weil OPwr offenbar wieder
//   aktiviert wird! Einmal Half-Shoot kurz nach LV-Ende behebt das Problem.
//
inline
bool IsIdle()
{
#if defined(EOS6Dii_MIRROR)		// nur Spiegel-Emu mit Hardware-Shutter?
  if (digitalRead(OPWR) == LOW) {	//  dann Coil-Signale nur akzeptieren, wenn Opt-Pwr aktiv ist
    return true;			//  sonst gibts bei Ready -> Idle aktive Flanken
  }
#endif
  return false;
}

//////////////////////////////////////////////////////////////////////////////////////
// Interrupt-Handler für Coil(1)
//
void trigger1_interrupt()
{
  if (IsIdle()) {	// Spiegel-Emu mit Hardware-Shutter?
    return;		//  Ready -> Idle macht aktive Flanken
  }
# if TRIGGER1_MODE == CHANGE	
  if (digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {    // Coil aktiv
    triggerTime = micros();
    triggerEvent = evCoil1Start;     // Flag für aktive Flanke
PRINT("C1_L ");
  }
  else
# endif
    if (actionPtr != 0			// Power-Up: Bei externer Versorgung bekommen wir das mit!
# if defined(TRIGGER2_MODE) && (defined(EOS650D_700D) || defined(EOS60D) || defined(EOS70D))
      // Auch Coil 2 Signale:  Dann beachten, dass bei der 60D/70D/650D/700D					
      //  im LV-Stop Zyklus C1_H zeitgleich zu C2_H kommt. Deshalb
      //  dieses C1_H nicht auswerten, um Überschneidungen zu vermeiden.
      &&  triggerEvent == evNone && actionPtr != c2_H_LeaveLV
      //  (Die 600D generiert in LeaveLV kein C2_H!)
# endif
    ) {
      triggerTime = micros();
# if defined(EOS6D)  ||  defined(EOS80D)
      // Beim LeaveLV generieren C1 und C2 identische Pulse von ca. 27 bzw. 21 ms, von denen
      //  einer - C1_H - unterdrückt werden muss.
      // Wenn C1_H zuerst kommt, liegt das letzte Ereignis ca. 27 bzw. 21 ms zurück und StartInLV wurde zuletzt ausgeführt.
      // Wenn C1_H 0..1ms nach C2_H kommt, ist entweder noch triggerEvent==evCoil2Stop oder schon actionPtr==c2_H_LeaveLV.
      // Um zu vermeiden, dass mit kurzen Belichtungen (< 1/1000s) Konflikte auftreten, muss genau geprüft werden.
      //
      unsigned long dt = triggerTime - timingZero;
      if (    (IsInWindow_LeaveLV(dt)  &&  actionPtr == c1_L_StartInLV) // erstes Ereignis nach C1_L?
	  ||  actionPtr == c2_H_LeaveLV	  // C2_H schon erkannt und bearbeitet?
	    // die folgende Bedingung tritt auch bei sehr kurzen Belichtungen auf und
	    //  führt dazu, das bei der 6Dii 1/3600 oder kürzer Err30 gibt!
	    //
	  ||  triggerEvent==evCoil2Stop	  // C2_H erfolgt, aber noch nicht in Arbeit? 
      ) {
	return;
      }
# endif
      triggerEvent = evCoil1Stop;      // Ende
PRINT("C1_H ");
    }
  //ToggleDebugLED();
}

#if defined(TRIGGER2_MODE)		// Auch Coil 2 Signale
//////////////////////////////////////////
// Interrupt-Handler für Coil 2 RISING
//
void trigger2_interrupt()
{
  if (IsIdle()) {	// Spiegel-Emu mit Hardware-Shutter?
    return;		//  Ready -> Idle macht aktive Flanken
  }

#if TRIGGER2_MODE == CHANGE	
  // Für die 6D übernimmt Coil_2 die Start-Aufgabe, weil C1 beim LV-Start inaktiv (HIGH) bleibt!
  //  Um die Ablauflogik nicht weiter zu verkomplizieren, wird ein evCoil1Start Ereignis
  //  ausgelöst, das ggf. auch von einem parallelen C1_H überlagert werden dürfte (das aber
  //  nicht durchgelssen wird). 
  //
  if (digitalRead(TRIGGER2) == TRIGGER2_ACTIVE) {    // Coil aktiv
    triggerTime = micros();
    triggerEvent = evCoil1Start;     // Flag für aktive Flanke (für die 6D: der Coil 1!) setzen
PRINT("C2_L ");
    return;
  }
# endif
  if (actionPtr == 0) return;					// Power-Up: Bei externer Versorgung bekommen wir das mit!
  triggerEvent = evCoil2Stop;
  triggerTime = micros();
  ToggleDebugLED();
  //BlinkLED(17 ms);
PRINT("C2_H ");
}
#endif


//
// Die seit dem Start des aktuellen Events vergangene Zeit in der Basiseinheit (aktuell µs) bestimmen.
//
unsigned long timeElapsed()
{
  // ACHTUNG:
  //  ms und µs werden als unsigned long bereitgestellt -- int sind hier nur 16 Bit breit!!!
  //  µs (von micros()) laufen schon nach 71 min über!
  //  Der Timer bleibt in allen Sleep-Modi außer SLEEP_MODE_IDLE stehen!!!!!!!!!!!
  //  Die Auflösung von micros() ist bei 8 Mhz nur 8µs (bzw. 4µs bei 16 MHz).
  //
  // Sinnvoll wäre die Umstellung der Basiszeiteinheit auf ms, weil die Shutter eh nicht genauer sind!
  //
  /*
  // Wenn die clock seit Start einen Überlauf hatte, ist ihr Wert ist jetzt um
  //  MAXULONG+1 (ca. 4295 sec) zu klein. Ebenso wie die Differenz zum Start.
  //  Dadurch ergibt sich bei der Differenzbildung ein Underflow, der das
  //  Problem behebt. Solange also innerhalb eines Zyklus nur ein
  //  Überlauf auftritt, gibt es kein Problem!
  //
  unsigned long t = micros() - timingZero;
  if ((long)t < 0) {				// Überlauf?
	  t -= MAXULONG+1;				// == 't -= 0' !!!!
  }
  return DebugSlowdown(t);
  */
  return DebugSlowdown(micros() - timingZero);
}


// Wert val an Pin pin ausgeben
void SetPin(int pin, int val)
{
  digitalWrite(pin, val);
  ToggleDebugLED();
}

#if defined(DEBUG_BLINK)
unsigned long nextBlink;

void BlinkLED(long n)
{
  if (blinkCount <= 0) BlinkLEDForced(n);
}

void BlinkLEDForced(long n)
{
  blinkCount = n / 1000;
  nextBlink = micros();
  digitalWrite(LED_BUILTIN, 0);
}

// Ggf. LED umschalten zum Blinken
//
// Rückgabe: true, wenn alles erledigt ist
//
bool DoBlinkLED()
{
  if (blinkCount <= 0) {
    return true;
  }
  //if (triggerEvent != evNone) return true;
  if (micros() < nextBlink) {
    return false;									// noch warten
  }
  bool nxt = !digitalRead(LED_BUILTIN);
  digitalWrite(LED_BUILTIN, nxt);
  if (blinkCount >= 100) {				// 100ms
    if (nxt) {									//  an
      nextBlink = micros() + 500000L;
    }
    else {
      nextBlink = micros() + 200000L;
      blinkCount -= 100;
      if (blinkCount < 100) nextBlink += 200000L;	// lange Pause
    }
    return false;
  }
  if (blinkCount >= 10) {				// Zehner
    if (nxt) {									//  an
      nextBlink = micros() + 300000L;
    }
    else {
      nextBlink = micros() + 200000L;
      blinkCount -= 10;
      if (blinkCount < 10) nextBlink += 200000L;	// lange Pause
    }
    return false;
  }
  if (blinkCount > 0) {				// Einer
    if (nxt) {									//  an
      nextBlink = micros() + 100000L;
    }
    else {
      blinkCount -= 1;
      nextBlink = micros() + 200000L;
    }
  }
  return blinkCount <= 0;		// fertig, wenn 0 erreicht
}
#endif


// Initialisierung beim Start (oder nach Reset)
void setup()
{
  // Ruhezustand der Pins init.
  stateIndex = state_Off;
  actionPtr = 0;
  SetPin(OPT1, HIGH);
# ifdef   OPT2
  SetPin(OPT2, HIGH);
# endif

  SetPin(ROT1, ROT1_INIT);
  SetPin(ROT2, ROT2_INIT);
# ifdef   ROT3
  SetPin(ROT3, ROT3_INIT);
# endif
# ifdef   ROT4
  SetPin(ROT4, ROT4_INIT);
# endif
# ifdef   ROTX
  SetPin(ROTX, ROTX_INIT);
# endif
# ifdef   MR1
  SetPin(MR1, MR1_INIT);
# endif
# ifdef   MR3
  SetPin(MR3, MR3_INIT);
# endif
# ifdef   MR2
  SetPin(MR2, MR2_INIT);
# endif

  //
  // Funktion der emulierten Pins (COIL1 mit Pullup, s.o.):
  pinMode(COIL1, INPUT_PULLUP);
#if defined(EOS6Dii_MIRROR) // 
  pinMode(OPWR, INPUT);	// Spiegel-Opt-Power Input statt Opt1 Output
#else
  pinMode(OPT1, OUTPUT);
#endif
  pinMode(ROT1, OUTPUT);
  pinMode(ROT2, OUTPUT);
#ifdef   OPT2
  pinMode(OPT2, OUTPUT);
#endif
#ifdef   COIL2
  pinMode(COIL2, INPUT_PULLUP);
#endif
#ifdef   MOTORDRV
  pinMode(MOTORDRV, INPUT);   // 80D
stime = micros();
#endif

#ifdef   ROT3
  pinMode(ROT3, OUTPUT);
#endif
#ifdef   ROT4
  pinMode(ROT4, OUTPUT);
#endif
#ifdef   ROTX
  pinMode(ROTX, OUTPUT);
#endif
#ifdef   MR1
  pinMode(MR1, OUTPUT);
#endif
#ifdef   MR2
  pinMode(MR2, OUTPUT);
#endif
#ifdef   MR3
  pinMode(MR3, OUTPUT);
#endif


#if defined(DEBUG_BLINK)
  blinkCount = 0;
#endif
#if  defined(DEBUG_SLOW) ||  defined(DEBUG_BLINK)
  pinMode(LED_BUILTIN, OUTPUT);
#endif

while (!IsIdle()  &&  digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {StartSerial();}
attachInterrupt(digitalPinToInterrupt(TRIGGER1), trigger1_interrupt, TRIGGER1_MODE);
#ifdef TRIGGER2_MODE
  while (!IsIdle()  &&  digitalRead(TRIGGER2) == TRIGGER2_ACTIVE) {StartSerial();}
  attachInterrupt(digitalPinToInterrupt(TRIGGER2), trigger2_interrupt, TRIGGER2_MODE);
#endif
  
  triggerEvent = evNone;
  StartSerial();
}
